@@ -1,7 +1,7 @@
 /** 
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 1997-2006  Gaspar Sinai <gaspar@yudit.org>  
+ *  GNU Copyright (C) 1997-2023  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -51,122 +51,10 @@ extern "C"
 #include <sys/types.h>
 #endif
 
+
+
 static SString sedyInit();
 
-
-#define TEST 0
-
-#define DIALOG_TEST 1
-#if TEST
-
-#if DIALOG_TEST
-
-#include <swidget/SFileDialog.h>
-#include <swidget/STextDialog.h>
-
-int
-main (int argc, char* argv[])
-{
-    SAwt awt;
-#if 1
-    STextDialog * dialog = new STextDialog();
-    dialog->setBackground(SColor("#7f7f7f"));
-    dialog->getInput ("Title", "This is an even longer\nessage", 
-        SDialog::SS_QUESTION);
-#endif
-#if 0 
-    SFileDialog * dialog = new SFileDialog();
-    dialog->setBackground(SColor("#7f7f7f"));
-    dialog->getInput ("Open", false, false);
-#endif
-/*
-    panel->resize (SDimension(520,300));
-    panel->show();
-    SEventHandler::start();
-*/
-    return (0);
-}
-
-#else /* DIALOG_TEST */
-
-#include <swidget/SPanel.h>
-#include <swidget/SIconFactory.h>
-#include <swindow/SImage.h>
-#include <stoolkit/SUniqueID.h>
-
-class MyPanel : public SPanel 
-{
-public:
-    MyPanel () {};
-    virtual ~MyPanel() {};
-
- void redraw(SCanvas* w, int x, int y,
-     unsigned int width ,unsigned int height)
-{
-    SStringVector list = SIconFactory::getIconList();
-#if 0
-    SIcon* icon = SIconFactory::getIcon(SString("EmbedLeft"));        
-    SImage image = icon->getImage();        
-    SVGBase* svgBase = image.getSVGBase();
-    svgBase->setCanvas (window, SColor("white"));
-    svgBase->render(25, 25, 1.0);
-    svgBase->render(85, 25, 2.0);
-    svgBase->render(190, 25, 4.0);
-#else
-//SUniqueID id;
-//fprintf (stderr, "Id [%*.*s]\n", SSARGS(id));
-    for (unsigned int i=0; i<list.size(); i++) {
-        SIcon* icon = SIconFactory::getIcon(list[i]);        
-        SImage image = icon->getImage();        
-        unsigned int x = 10 + (i%10) * 50;
-        unsigned int y = 10 + (i/10) * 50;
-        window->putImage (x, y, image);
-        SVGBase* svgBase = image.getSVGBase();
-        if (svgBase) {
-            svgBase->setCanvas (window, SColor("#7f7f7f"));
-            svgBase->render(x + 25, y);
-        }
-    }
-#endif
-}
-
- void redraw2(SCanvas* w, int x, int y,
-     unsigned int width ,unsigned int height)
-
-{
-    if (!window->beginImage (0, 0, "Image0", SColor("white"))) {
-        fprintf (stderr, "Image"); 
-        window->newpath (); 
-        window->moveto (200,100); 
-        window->lineto (100,200); 
-        window->lineto (300,200); 
-        window->closepath (); 
-        SPen pen(SColor("yellow"), SColor("white"));
-        window->fill (pen);
-        pen.setLineWidth (1.0);
-        pen.setForeground (SColor("black"));
-        window->stroke (pen);
-        
-    }
-    window->endImage();
-}
-};
-
-
-int
-main (int argc, char* argv[])
-{
-    SAwt awt;
-    MyPanel * panel = new MyPanel();
-    panel->setBackground(SColor("#7f7f7f"));
-    panel->resize (SDimension(520,300));
-    panel->show();
-    SEventHandler::start();
-    return (0);
-}
-#endif
-#else
-
 int
 main (int argc, char* argv[])
 {
@@ -240,6 +128,7 @@ main (int argc, char* argv[])
    */
   //properties.put ("yudit.datapath", );
   //properties.put ("yudit.fontpath", );
+  properties.put ("yudit.scale", "1.0");
   properties.put ("yudit.background", "gray50");
   properties.put ("yudit.label.foreground", "moccasin");
   properties.put ("yudit.title.foreground", "white");
@@ -427,8 +316,7 @@ main (int argc, char* argv[])
     if (!awt.hasGUI()) 
     {
       fprintf (stderr, "AWT is not implemented on this platform.\n");
-      fprintf (stderr, "This can happen if X11 Development libraries\n"); 
-      fprintf (stderr, "were missing when configure was executed.\n"); 
+      fprintf (stderr, "This can happen if './configure --without-gui' was used before compilation.\n"); 
       fprintf (stderr, "Other Yudit programs (uniconv, uniprint) should work.\n"); 
       return 2;
     }
@@ -519,6 +407,26 @@ main (int argc, char* argv[])
   }
 
   SString sedyLib = sedyInit ();
+  SString scaleStr = allProperties["yudit.scale"];
+  scaleStr.append ((char)0);
+  float scale = 1.0;
+  char buff[8];
+  if (sscanf (scaleStr.array(), "%f%1s", &scale, buff) != 1) 
+  {
+    fprintf (stderr, "Yudit: decimal number expected for [yudit.scale=%s]\n", 
+        scaleStr.array());
+    scale = 1.0;
+  }
+  if (scale > SB_YUDIT_MAX_SCALE) {
+    fprintf (stderr, "Yudit: [yudit.scale=%s] not in range [1.0-%.1f]\n", scaleStr.array(), SB_YUDIT_MAX_SCALE);
+    scale = 3.0;
+  }
+  if (scale < 1.0) {
+    fprintf (stderr, "Yudit: [yudit.scale=%s] not in range [1.0-%.1f]\n", scaleStr.array(), SB_YUDIT_MAX_SCALE);
+    scale = 1.0;
+  }
+  SAwt::setScale (scale);
+
   SYudit *yudit = new SYudit (args, allProperties, sedyLib);
   yudit->show();
   yudit->center();
@@ -536,11 +444,18 @@ static SString sedyInit () {
   v.append (c2);
   SString lib = SSedy::loadLibrary (v);
   if (lib.size() != 0) {
-//    fprintf (stderr, "Sedy: %*.*s\n", SSARGS(lib));
+     // fprintf (stderr, "Sedy: %*.*s\n", SSARGS(lib));
+    SString test = "Test String...";
+    SString pass = "VeryGoodPassword";
+    SSedy sedy;
+    SString enc = sedy.encrypt (test, pass);
+    SString dec = sedy.decrypt (enc, pass);
+    if (dec != test) {
+        fprintf (stderr, "Sedy: %*.*s failed test.\n", SSARGS(lib));
+    }
+    //return SString("");
   }
   //SSedy sedy;
   //fprintf (stderr, "Sedy: %*.*s\n", SSARGS(sedy.getLastError()));
   return SString(lib);
 }
-
-#endif
