@@ -1,7 +1,7 @@
 /** 
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 1997-2006  Gaspar Sinai <gaspar@yudit.org>  
+ *  GNU Copyright (C) 1997-2023  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -21,6 +21,7 @@
 
 #include "gui/Main.h"
 #include "swidget/SIconFactory.h"
+#include "stoolkit/STypes.h" 
 
 SToolBarListener::SToolBarListener(void)
 {
@@ -171,7 +172,7 @@ SToolBar::SToolBar (const SProperties props)
     buttons[i]->resize(wp);
     buttons[i]->move(SLocation ((int)p, wp.height));
     add (buttons[i]);
-    p+= wp.width + 2;
+    p+= wp.width + 2 *SAwt::getRasterScale();
     if (wp.height > h) h = wp.height;
   }
 
@@ -226,10 +227,11 @@ SToolBar::inputChanged (const SStringVector& _inputs, unsigned int _current)
   inputs = _inputs;
   currentInput = _current;
   buildDisplayInputs();
-  
+
   SDimension wp = getMaxSize (cb, displayInputs, displayInputs[currentInput]);
   int delta = (int) wp.width - (int) oldSize.width;
   unsigned int i;
+  delta = 1;
   if (delta != 0)
   {
     cb->resize (SDimension ((int)oldSize.width + delta, oldSize.height));
@@ -360,6 +362,7 @@ SToolBar::setFileName (const SString& _filename, const SString& _encoding)
 void
 SToolBar::resize(const SDimension & d)
 {
+//fprintf (stderr, "SToolBar After Resize x=%d\n", buttons[SToolBar::SS_PARAGRAPH_BREAK]->getLocation().x);
   SPanel::resize(d);
   window->redraw (true, title->getLocation().x, 
      title->getLocation().y, title->getSize().width, title->getSize().height);
@@ -415,6 +418,7 @@ SToolBar::resize(const SDimension & d)
     }
     buttons[i]->resize(wp);
   }
+//fprintf (stderr, "SToolBar After Resize x=%d\n", buttons[SToolBar::SS_PARAGRAPH_BREAK]->getLocation().x);
 }
 
 void
@@ -588,18 +592,19 @@ SToolBar::enterComponent (void* source)
         char aa[64];
         if (max > 1)
         {
-           sprintf (aa, "%s(__%sF1..F%d__)%s", SS_LB_LRE,
+           snprintf (aa, 64, "%s(__%sF1..F%d__)%s", SS_LB_LRE,
              (i==(unsigned int)SS_FONT) ? "shift-" : "",
              (max), SS_LB_PDF);
         }
         else
         {
-           sprintf (aa, "%s(__%sF1__)%s", SS_LB_LRE, 
+           snprintf (aa, 64, "%s(__%sF1__)%s", SS_LB_LRE, 
              (i==(unsigned int)SS_FONT) ? "shift-" : "", SS_LB_PDF);
         }
         str.append (SString(aa));
       }
       SLocation loc = buttons[i]->getLocation ();
+//fprintf (stderr, "XXX Location=%d\n", loc.x);
       tooltip->setText (str);
       SDimension ld=tooltip->getPreferredSize();
 
