@@ -1,7 +1,7 @@
 /** 
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 1997-2006  Gaspar Sinai <gaspar@yudit.org>  
+ *  GNU Copyright (C) 1997-2023  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -131,7 +131,7 @@ main (int argc, char* argv[])
    * This option puts everything in a tmp file and 
    * calls the program with agruents specified after cmd, 
    * with the filename as the last argument.
-   * when porgram exists remove the tmp file.
+   * when program exists remove the tmp file.
   */
   if (argc>=3 && strcmp ("-pipecmd", argv[1])==0)
   { 
@@ -190,7 +190,7 @@ main (int argc, char* argv[])
 
     /* wait till it exits */
     int ret = pipe.wait();
-    /* remove tmp file after it openend it  */
+    /* remove tmp file after it opened it  */
 #ifdef USE_WINAPI
     Sleep(10 * 1000);
 #else
@@ -507,7 +507,7 @@ main (int argc, char* argv[])
     if (source_date_epoch == NULL || (tim = (time_t)strtol (source_date_epoch, NULL, 10)) <= 0)
       tim = time(0);
     struct tm* ts = localtime (&tim);
-    sprintf (shortComment, "created by mytool %04d-%02d-%02d", 
+    snprintf (shortComment, MAX_COMMENT_SIZE, "created by mytool %04d-%02d-%02d", 
       ts->tm_year+1900, ts->tm_mon+1, ts->tm_mday);
     map.setComment ((const unsigned char*) shortComment,
       strlen (shortComment));
