@@ -1,7 +1,7 @@
 /** 
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 1997-2006  Gaspar Sinai <gaspar@yudit.org>  
+ *  GNU Copyright (C) 1997-2023  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -113,7 +113,7 @@ static int write1(SString* _fd, const unsigned char* out, int size, SFileFormat
 /**
  * SBMap can be used to map any N-word-long string into 
  * any n word long string. The supported word sizes are 1,2,4 and 8.
- * The array of words can be arbirary long.
+ * The array of words can be arbitrary long.
  */
 SBMap::SBMap ()
 {
@@ -158,15 +158,16 @@ SBMap::setName (const unsigned char* _name)
   nameType = SBMap_DYNAMIC;
   nameBuffer = new unsigned char [32];
   CHECK_NEW (nameBuffer);
-  strncpy ((char*) nameBuffer, (const char*) _name, 32);
+  memset (nameBuffer, 0, 32);
+  strncpy ((char*) nameBuffer, (const char*) _name, 31);
   name = nameBuffer;
   status = true;
 }
 
 /**
  * Set the comment.
- * @param _commment is a comment of arbitrary length.
- * @param _commmentSize is the length of the commnent.
+ * @param _comment is a comment of arbitrary length.
+ * @param _commentSize is the length of the comment.
  */
 void
 SBMap::setComment (const unsigned char* _comment, int _commentSize)
@@ -187,7 +188,7 @@ SBMap::~SBMap()
 }
 
 /**
- * Addd a new item to the SBMapBuffer. You can add an SBMapItem only if the 
+ * Add a new item to the SBMapBuffer. You can add an SBMapItem only if the 
  * SBMap has been properly initialized.
  * @param _item is the item to add. This will be owned by SBMap.
  * @param _position is the position of the SBMapItem, if it is less than 0 
@@ -234,7 +235,7 @@ SBMapBuffer::~SBMapBuffer()
 }
 
 /**
- * Append an aray of input and put it in the internal buffer.
+ * Append an array of input and put it in the internal buffer.
  * @param buffer is a pointer to the input array of inWordSize bytes type
  * @param _length is the length of the input array of specified type
  */
@@ -346,7 +347,7 @@ SBMapBuffer::shift (int _size)
 
 
 /**
- * Append an aray of input and put it in the internal buffer.
+ * Append an array of input and put it in the internal buffer.
  * @param buffer is a pointer to the input array of inWordSize bytes type
  *        This is in native order.
  * @param _length is the length of the input array of specified type
@@ -487,7 +488,7 @@ SBMapBuffer::appendToNet(const void *buffer, int _length)
 }
 
 /**
- * Append an aray of input of net order to the internal buffer.
+ * Append an array of input of net order to the internal buffer.
  * @param buffer is a pointer to the input array of inWordSize bytes type
  *     This is in Net order.
  * @param _length is the length of the input array of specified type
@@ -1370,7 +1371,7 @@ SBMapItem::SBMapItem (int _encode,
   nameBuffer = new unsigned char[32];
   CHECK_NEW (nameBuffer);
   memset (nameBuffer, 0, 32);
-  strncpy ((char*)nameBuffer, (const char*) _name, 32);
+  strncpy ((char*)nameBuffer, (const char*) _name, 31);
   nameType = SS_DYNAMIC;
   name = nameBuffer;
 
@@ -1405,7 +1406,7 @@ SBMapItem::SBMapItem (int _encode,
 
 SBMapItem::~SBMapItem ()
 {
-//fprintf (stderr, "Deleteing map dinamic=%d this=%lx name=%s\n",
+//fprintf (stderr, "Deleting map dynamic=%d this=%lx name=%s\n",
 // stateMachineType == SS_DYNAMIC, (unsigned long) this, nameBuffer);
   if (itemType == SBMapNToN && stateMachineType == SS_DYNAMIC)
   {
@@ -1449,7 +1450,7 @@ SBMapItem::strip ()
 
 /**
  * return the comment.
- * @param at is the index at wich the string is
+ * @param at is the index at which the string is
  * @param size is the byte-length of string.
  */
 const unsigned char*
@@ -1497,7 +1498,7 @@ SBMapItem::getComment (int at, unsigned int *size)
 
 /**
  * get the length of this item.
- * @param at is the index at wich the string is
+ * @param at is the index at which the string is
  * @return the length of this cell.
  */
 unsigned int
@@ -1590,19 +1591,19 @@ SBMapItem::convertFromBumap()
     if (output == 0) continue;
     if (inWordSize==0)
     {
-      sprintf (fullLine, "%02X -> ", input&0xff);
+      snprintf (fullLine, 64, "%02X -> ", input&0xff);
     }
     else
     {
-      sprintf (fullLine, "%04X -> ", input&0xffff);
+      snprintf (fullLine, 64, "%04X -> ", input&0xffff);
     }
     if (outWordSize==0)
     {
-      sprintf (word, "%02X", output&0xff);
+      snprintf (word, 64, "%02X", output&0xff);
     }
     else
     {
-      sprintf (word, "%04X", output&0xffff);
+      snprintf (word, 64, "%04X", output&0xffff);
     }
     strcat (fullLine, word);
     status = addLine ((const unsigned char*) fullLine, strlen (fullLine));
@@ -1833,7 +1834,7 @@ SBMapItem::nextSorted (const unsigned char* key, unsigned int klen)
 
 /**
  * Add a new line into the map. Clear state machine, and make map dynamic.
- * @param _line is the ponter to the buffer for input
+ * @param _line is the pointer to the buffer for input
  *   the input line should look like tthis:
  *   <pre>
  *   20 23 6f 45  -> aa9c
@@ -1958,8 +1959,8 @@ SBMapItem::addLine (const unsigned char* _line, unsigned int _length, bool _reve
 /**
  * convert nibbles into a real unsgined char string. This string is owned by 
  * calling function.
- * @param in is the input charater buffer. 
- * @param _size is the size of the input charater buffer. 
+ * @param in is the input character buffer. 
+ * @param _size is the size of the input character buffer. 
  * @param _len is the length of the resulting array.
  * @param _match is the length of the buffer till '/' 
  * @return the converted hexadecimal buffer.
@@ -2052,7 +2053,7 @@ SBMapItem::toHex (const unsigned char* in, unsigned int _size, unsigned int* _le
 
 /**
  * return value byte-string from index.
- * @param at is the index at wich the string is
+ * @param at is the index at which the string is
  * @param size is the byte-length of string.
  */
 const unsigned char*
@@ -2096,7 +2097,7 @@ SBMapItem::getValue (int at, unsigned int *size)
 
 /**
  * return value byte-string from index.
- * @param at is the index at wich the string is
+ * @param at is the index at which the string is
  * @param size is the byte-length of string.
  */
 const unsigned char*
@@ -2319,7 +2320,7 @@ SBMapItem::find(const unsigned char in, unsigned int pos, int* from, int* to)
 
 /**
  * Get the name of the specified map
- * @param line the buffer to strore the string. This will be a 
+ * @param line the buffer to store the string. This will be a 
  *        full string with no chars of 0x00 replaced by a space.
  * @param len the length of the buffer.
  * @param mapIndex the index of the map. If a number that is less than zero
@@ -2339,7 +2340,7 @@ SBMap::getName (char* line, int len, int mapIndex)
 
 /**
  * Get the name of the specified map
- * @param line the buffer to strore the string. This will be a 
+ * @param line the buffer to srore the string. This will be a 
  *        full string with no chars of 0x00 replaced by a space.
  * @param len the length of the buffer.
  * @param mapIndex the index of the map. If a number that is less than zero
@@ -2399,7 +2400,7 @@ SBMap::packString (char* line, int len, const unsigned char* input, int maxlen)
 }
 
 /**
- * buld a new state machine.
+ * build a new state machine.
  * @param the map to build the state machine for, -1 meas all.
  */ 
 void
@@ -2873,7 +2874,7 @@ SBMapItem::addState(SS_WORD32 oldState, const unsigned char in, unsigned int pos
   {
     from = _from; to = _to;
 
-    // If divisable by 2 it means that this is the 
+    // If divisible by 2 it means that this is the 
     // second nibble and we may have to do some work...
     // Determine what STT to pass. That can be MATCH,
     // REJECT or MORE.
@@ -3130,7 +3131,7 @@ SBMapItem::serialize (SString* _fd, SFileFormat _format, int _last)
 
 /**
  * write a formatted text with a length.
- * @param _fd is the file descriptior to write to
+ * @param _fd is the file descriptor to write to
  * @param _buf is the buffer tow write.
  * @param _length is the length in bytes (not in words!)
  * @param _slash is the length where slash is needed.
@@ -3231,7 +3232,7 @@ SBMapItem::writeTextBytes (SString* _fd, const unsigned char *_buf,
 }
 
 /**
- * write out the code area accroding to format.
+ * write out the code area according to format.
  * @param _fd is the output file descriptor
  * @param _index is the array index of the code area
  * @param _format is the file format  SS_BINARY, SS_TEXT_MAP, SS_CTEXT
@@ -3486,11 +3487,11 @@ write8 (SString* _fd, SS_WORD8 number, SFileFormat _format, const SS_FieldType f
     pref=textMapPrefix[ftype];
     if (number > ' ' && number < 'z' && number != '#')
     {
-      sprintf ((char*)out, "'%c", number);
+      snprintf ((char*)out, 64, "'%c", number);
     }
     else
     {
-      sprintf ((char*)out, "%02X", number);
+      snprintf ((char*)out, 64, "%02X", number);
     }
     if (lend==SS_LAST || lend==SS_EOF)
     {
@@ -3516,15 +3517,15 @@ write8 (SString* _fd, SS_WORD8 number, SFileFormat _format, const SS_FieldType f
     pref=cMapPrefix[ftype];
     if (lend == SS_LAST)
     {
-      sprintf ((char*)out, "0x%02x,\n", number);
+      snprintf ((char*)out, 64, "0x%02x,\n", number);
     }
     else if (lend == SS_EOF)
     {
-      sprintf ((char*)out, "0x%02x\n", number);
+      snprintf ((char*)out, 64, "0x%02x\n", number);
     }
     else
     {
-      sprintf ((char*)out, "0x%02x, ", number);
+      snprintf ((char*)out, 64, "0x%02x, ", number);
     }
     if (writeBytes (_fd, (const unsigned char*) pref, strlen(pref)) < 0)
     {
@@ -3565,7 +3566,7 @@ write16 (SString* _fd, SS_WORD16 number, SFileFormat _format, const SS_FieldType
   case SS_TEXT_MAP:
     if (ftype == FT_IGNORE) return 2;
     pref=textMapPrefix[ftype];
-    sprintf ((char*)out, "%04X", number);
+    snprintf ((char*)out, 64, "%04X", number);
     if (lend == SS_LAST || lend == SS_EOF)
     {
       strcat ((char*)out, "\n");
@@ -3592,17 +3593,17 @@ write16 (SString* _fd, SS_WORD16 number, SFileFormat _format, const SS_FieldType
     pref=cMapPrefix[ftype];
     if (lend == SS_LAST)
     {
-      sprintf ((char*)out, "0x%02x, 0x%02x,\n", 
+      snprintf ((char*)out, 64, "0x%02x, 0x%02x,\n", 
         (int) num[0], (int) num[1]);
     }
     else if (lend ==SS_EOF)
     {
-      sprintf ((char*)out, "0x%02x, 0x%02x\n",
+      snprintf ((char*)out, 64, "0x%02x, 0x%02x\n",
         (int) num[0], (int) num[1]);
     }
     else
     {
-      sprintf ((char*)out, "0x%02x, 0x%02x, ", 
+      snprintf ((char*)out, 64, "0x%02x, 0x%02x, ", 
         (int) num[0], (int) num[1]);
     }
     if (writeBytes (_fd, (const unsigned char*) pref, strlen(pref)) < 0)
@@ -3645,7 +3646,7 @@ write32 (SString* _fd, SS_WORD32 number, SFileFormat _format, const SS_FieldType
   case SS_TEXT_MAP:
     if (ftype == FT_IGNORE) return 4;
     pref=textMapPrefix[ftype];
-    sprintf ((char*)out, "%08X", number);
+    snprintf ((char*)out, 64, "%08X", number);
     if (lend == SS_LAST || lend == SS_EOF)
     {
       strcat ((char*)out, "\n");
@@ -3674,19 +3675,19 @@ write32 (SString* _fd, SS_WORD32 number, SFileFormat _format, const SS_FieldType
     num[3] = ((number >> 0) & 0xff);
     if (lend == SS_LAST)
     {
-      sprintf ((char*)out, "0x%02x, 0x%02x, 0x%02x, 0x%02x,\n", 
+      snprintf ((char*)out, 64, "0x%02x, 0x%02x, 0x%02x, 0x%02x,\n", 
         (int) num[0], (int) num[1],
         (int) num[2], (int) num[3]);
     }
     else if (lend == SS_EOF)
     {
-      sprintf ((char*)out, "0x%02x, 0x%02x, 0x%02x, 0x%02x\n", 
+      snprintf ((char*)out, 64, "0x%02x, 0x%02x, 0x%02x, 0x%02x\n", 
         (int) num[0], (int) num[1],
         (int) num[2], (int) num[3]);
     }
     else
     {
-      sprintf ((char*)out, "0x%02x, 0x%02x, 0x%02x, 0x%02x,", 
+      snprintf ((char*)out, 64, "0x%02x, 0x%02x, 0x%02x, 0x%02x,", 
         (int) num[0], (int) num[1],
         (int) num[2], (int) num[3]);
     }
@@ -3788,11 +3789,11 @@ SBMapItem::getDecoderMap (SStringVector* key, SStringVector* value,
         ucs4vout.append (u4out);
         if (in <= 0xff)
         {
-           sprintf (kc, "=%02X", (unsigned int)in);
+           snprintf (kc, 32, "=%02X", (unsigned int)in);
         }
         else
         {
-           sprintf (kc, "=%04X", (unsigned int)in);
+           snprintf (kc, 32, "=%04X", (unsigned int)in);
         }
         SString ks(kc);
         SString vs = _encoder.encode (ucs4vout);
