@@ -1,7 +1,7 @@
 /** 
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 1997-2006  Gaspar Sinai <gaspar@yudit.org>  
+ *  GNU Copyright (C) 1997-2023  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -136,8 +136,10 @@ SParagraph::expand() const // Well, not....
 
   if (end != chrs.size()) 
   {
-    fprintf (stderr, "SParagraph::expand() internal error\n");
+    fprintf (stderr, "SParagraph::expand() internal error end=%d, size=%d\n", 
+        end, chrs.size());
   }
+
   p->ucs4Glyphs.clear();
   p->expanded = true;
   if (selected)
@@ -163,6 +165,16 @@ SParagraph::expand() const // Well, not....
   p->clearChange();
   p->reordered = true;
   p->reShape (); 
+
+  /* Yudit may add 202C before end of paragraph to fix 
+     unmatched embedding and override
+     U+202E u+2041 U+000A 
+     becomes 
+     U+202E u+0041 202C U+000A 
+     It also can get rid of unpaired ones.
+     therefore getChars().size() sometimes != chrs.size();
+   */
+
 }
 
 /**
@@ -268,7 +280,7 @@ SParagraph::reShape (unsigned int index)
 
 /**
  * Get the first non-spacing glyph left visual order in array, 
- * starting here and incuding this.
+ * starting here and including this.
  * return the index or -1 if not found.
  */
 int 
@@ -678,7 +690,7 @@ SParagraph::isProperLine () const
 }
 
 /*
- * size withoug EOL
+ * size without EOL
  */
 unsigned int
 SParagraph::properSize() const
@@ -865,20 +877,20 @@ SParagraph::setParagraphSeparator (SS_ParaSep ps)
   {
   case SS_PS_LF:
     v.append (SD_CD_LF);
-    newg = SGlyph(v, SD_CD_LF, false, 0, 0);
+    newg = SGlyph(v, SD_CD_LF, false, 0, 0, false);
     break;
   case SS_PS_CR:
     v.append (SD_CD_CR);
-    newg = SGlyph(v, SD_CD_CR, false, 0, 0);
+    newg = SGlyph(v, SD_CD_CR, false, 0, 0, false);
     break;
   case SS_PS_CRLF:
     v.append (SD_CD_CR);
     v.append (SD_CD_LF);
-    newg = SGlyph(v, 0, false, 0, 0);
+    newg = SGlyph(v, 0, false, 0, 0, false);
     break;
   case SS_PS_PS:
     v.append (SD_CD_PS);
-    newg = SGlyph(v, SD_CD_PS, false, 0, 0);
+    newg = SGlyph(v, SD_CD_PS, false, 0, 0, false);
     break;
   default:
     return false;
@@ -982,14 +994,14 @@ SParagraph::underline (bool is, unsigned int from, unsigned int to)
  * Split a text into Glyphs
  * @param ucs4 is a text that can contain paragraph separators
  * @param from is the starting index.
- * @parem gl is the return Glyphs.
+ * @param gl is the return Glyphs.
  * @return ending index, that is equal to starting index 
- * if ther is no more data.
+ * if there is no more data.
  */
 static unsigned int
 split (const SV_UCS4& ucs4, SVector<SGlyph>* gl, unsigned int from)
 {
-  SUniMap composer ("precompose");
+  SUniMap composer ("precompose1");
   SUniMap shaper ("shape");
 
   gl->clear();
@@ -999,7 +1011,6 @@ split (const SV_UCS4& ucs4, SVector<SGlyph>* gl, unsigned int from)
   bool    isShaped;
   bool    isLineEnd;
   //gl->ensure (ucs4.size());
-  SGlyphShared* shr = 0;
   unsigned int compIndex = 0;
 
   SV_UCS4   stack;
@@ -1015,7 +1026,6 @@ split (const SV_UCS4& ucs4, SVector<SGlyph>* gl, unsigned int from)
     isLineEnd = true;
     unsigned int clusterIndex = 0;
     SV_UCS4 du4;
-    shr = 0;
     switch (ucs4[i])
     {
     case SD_CD_RLO:
@@ -1038,6 +1048,10 @@ split (const SV_UCS4& ucs4, SVector<SGlyph>* gl, unsigned int from)
     case SD_CD_PS:
       du4.append (ucs4[i++]);
       break ;
+    case SD_CD_FF:
+      du4.append (ucs4[i++]);
+      // Prevent adding composing characters to it.
+      break;
     default:
       isLineEnd = false;
       if (shaper.isOK())
@@ -1092,7 +1106,7 @@ split (const SV_UCS4& ucs4, SVector<SGlyph>* gl, unsigned int from)
                    /* add for all shapes */
                    SV_UCS4 cm; cm.append (ret2[0]); cm.append (comb);
                    /* FIXME: the first parameter should be memory 
-                      representation. We are sloppy because noone is using it.*/
+                     representation. We are sloppy because no one is using it.*/
                    SS_UCS4 l0 = (cm[0]==0) ? 0 : addCombiningLigature (
                       cm.array(), cm.size(), cm.array(), cm.size());
 
@@ -1230,22 +1244,6 @@ split (const SV_UCS4& ucs4, SVector<SGlyph>* gl, unsigned int from)
         }
         else /* it was not a composition */
         {
-          /* quick get on a precomposed character */
-          shr = getGlyphShared (ucs4[i]);
-          if (shr != 0)
-          {
-            /*
-             * Glyphcache can have precomposed or single chars only. 
-             * But these single chars have have some comps...
-             */
-            du4 = shr->ucs4v;
-            /* we are saving memory in sglyph */
-            if (du4.size()==0) du4.append (ucs4[i]);
-            composition = shr->precomposed; /* ucs4[i] */
-            usePrecomposed = true;
-            i++;
-            break;
-          }
           ret.clear();
 
           /* check if it can be broken down into a composition */
@@ -1306,7 +1304,7 @@ split (const SV_UCS4& ucs4, SVector<SGlyph>* gl, unsigned int from)
          * 2. We can not merge <base><comp1> for shapes because base 
          *    is used for shaping. We must set compIndex after base
          *    for shaped, before base for unshaped glyphs. This
-         *    is just a convention here. For unshaped we don't realy
+         *    is just a convention here. For unshaped we don't really
          *    care: XXXX YYYY ZZZZ or XXXX ; YYYY ZZZZ. So we use former.
          */
         if (isShaped) 
@@ -1325,12 +1323,6 @@ split (const SV_UCS4& ucs4, SVector<SGlyph>* gl, unsigned int from)
             usePrecomposed = false;
             compIndex =0; /* also works with 1 - #2 is very weak */
           }
-          else if (du4.size()==1) /* make this a non-composing */
-          {
-            composition = 0;
-            usePrecomposed = false;
-            compIndex =0; /* also works with 1 - #2 is very weak */
-          }
           else /* composing extras */
           {
             compIndex = du4.size();
@@ -1345,14 +1337,32 @@ split (const SV_UCS4& ucs4, SVector<SGlyph>* gl, unsigned int from)
         }
       }
     }
-    SGlyph g (du4, composition, isShaped, clusterIndex, compIndex);
-    g.usePrecomp = (usePrecomposed || du4.size()==1);
+
+    bool realUsePrecomposed = usePrecomposed; 
+    if (du4.size()==1) {
+        if (composition == 0) {
+            composition = du4[0];
+        }
+        if (composition == du4[0])
+        {
+            du4.clear();
+            compIndex = 0;
+            realUsePrecomposed = true;
+        }
+    } 
+    if (composition == 0 && compIndex == 1) {
+        compIndex = 0;
+        realUsePrecomposed = false;
+    } 
+    SGlyph g (du4, composition, isShaped, clusterIndex, compIndex, realUsePrecomposed);
     if (!g.isEOL() && !g.isEOP())
     {
       g.setEmbeddingMarks (stack);
     }
     gl->append (g);
-    if (g.isEOP()) break;
+    if (g.isEOP()) {
+        break;
+    }
   }
   return i;
 }
