@@ -1,7 +1,7 @@
 /** 
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 1997-2006  Gaspar Sinai <gaspar@yudit.org>  
+ *  GNU Copyright (C) 1997-2023  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -204,7 +204,7 @@ getReg (const SString& _base, const SString& _name)
 }
 
 /* 
- * we dont have administrator privilage so this myght get virtualized.
+ * we dont have administrator privilege so this myght get virtualized.
  */
 static bool 
 setReg (const SString& _base, const SString& _name, const SString& _vle)
@@ -369,9 +369,10 @@ loadProperties (SProperties* in)
   SString c2 = getPrefix();
   c2.append ("/config/yudit.properties");
 
-  bool ret = loadProperties (c2, in);
-  ret = ret || loadProperties (c1, in);
-  return ret;
+  bool ret2 = loadProperties (c2, in);
+  // override with home
+  bool ret1 = loadProperties (c1, in);
+  return ret1 | ret2;
 }
 
 bool
@@ -471,7 +472,7 @@ SString translate (const SString& str)
     } 
     SString s (currentBind);
     s.append (".mo");
-    SFile file = SFile (s, plist, false);
+    SFile file = SFile (s, plist);
     moFileImage = file.getFileImage();
   }
   if (moFileImage.size()<=0) return SString(str);
@@ -589,15 +590,15 @@ unicodeValueOf (const SV_UCS4& ucs4)
   for (unsigned int i=0; i<ucs4.size(); i++)
   {
     if (ucs4[i] <= 0xffff) {
-        sprintf (a, "%04X", (unsigned int) ucs4[i]);
+        snprintf (a, 64, "%04X", (unsigned int) ucs4[i]);
     } else if (ucs4[i] <= 0xfffff) {
-        sprintf (a, "%05X", (unsigned int) ucs4[i]);
+        snprintf (a, 64, "%05X", (unsigned int) ucs4[i]);
     } else if (ucs4[i] <= 0xffffff) {
-        sprintf (a, "%06X", (unsigned int) ucs4[i]);
+        snprintf (a, 64, "%06X", (unsigned int) ucs4[i]);
     } else if (ucs4[i] <= 0xfffffff) {
-        sprintf (a, "%07X", (unsigned int) ucs4[i]);
+        snprintf (a, 64, "%07X", (unsigned int) ucs4[i]);
     } else {
-        sprintf (a, "%08X", (unsigned int) ucs4[i]);
+        snprintf (a, 64, "%08X", (unsigned int) ucs4[i]);
     }
     if (i!=0) s.append (" ");
     s.append (a);
@@ -625,8 +626,8 @@ ss_sqrtlong (unsigned long sq)
   /* we don't check for negative. assume...*/
   if (sq<2) return sq; /* zero or 1 */
 
-  register unsigned long x=sq;
-  register unsigned long r=x-1;
+  unsigned long x=sq;
+  unsigned long r=x-1;
   while (r < x)
   {
       x = r; r= (x+(sq/x))/2;
@@ -634,10 +635,10 @@ ss_sqrtlong (unsigned long sq)
   return x;
 }
 /**
- * arcus tanget in a scale of 32 
+ * arcus tangent in a scale of 32 
  * @param x is the vector x param
  * @param y is the vector y param
- * @return  the anlge with step 1
+ * @return  the angle with step 1
  * 0 => 12:00, 8 => 3:00, 16 => 6:00, 24 => 9:00
  * returns 32 if x and y are both zero.
  */
@@ -663,7 +664,7 @@ ss_atan32 (int x, int y)
   else if  (slope < 83) ret = 3;
   /* now 0..45 degrees is nicely split up into 0..4 */
 
-  /* go back the transform we did at the begining in reverse order */
+  /* go back the transform we did at the beginning in reverse order */
   if (xyflip) ret = 8 - ret; 
   if (yneg) ret = 16 - ret; 
   if (xneg) ret = 32 - ret;
@@ -924,3 +925,59 @@ SString utf8ToSystem (const SString& str) {
   return SString (ret);
 #endif
 }
+
+SStringVector getUserPath (const SString& property, const SString& directory) {
+  SStringVector outDataPath;
+
+  SString homeDir = getHome();
+  homeDir.append ("/");
+  SString c1 = homeDir;
+  c1.append (".yudit/");
+  c1.append (directory);
+  outDataPath.append (c1);
+
+  SProperties p;
+  loadProperties (&p);
+  if (p.get (property))
+  {
+// C: drive hack
+#ifdef USE_WINAPI
+      SStringVector v(p[property], ",;");
+#else
+      SStringVector v(p[property], ",:;");
+#endif
+      for (unsigned int j=0; j<v.size(); j++)
+      {
+         SString vle = v[j];
+         if (vle.size() == 0) continue;
+         if (vle.find ("~/") == 0) {
+            vle.replace("~/", homeDir);
+         }
+         outDataPath.append (vle);
+      }
+  }
+  SString c2 = getPrefix();
+  c2.append ("/");
+  c2.append (directory);
+  outDataPath.append (c2);
+  return SStringVector(outDataPath);
+} 
+
+SString getDocDir ()
+{
+    SString ret (getPrefix());
+    ret.append ("/doc");
+#if 0
+    SDir dir(ret);
+    if (!dir.readable()) {
+        SString fallback("/usr/share/doc/yudit");
+        fprintf (stderr, "Can not read: %*.*s Trying: %*.*s\n",   
+            SSARGS(ret), SSARGS(fallback));
+        SDir dir(fallback);
+        if (dir.readable()) {
+            ret = fallback;
+        }
+    }
+#endif
+    return SString(ret);
+}
