@@ -1,7 +1,7 @@
 /** 
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 1997-2006  Gaspar Sinai <gaspar@yudit.org>  
+ *  GNU Copyright (C) 1997-2023  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -77,7 +77,7 @@ SHunspellPattern::SHunspellPattern (const SString& _name,
 
   dicFile.append ((char)0);
   affFile.append ((char)0);
-  SString mising = loadLibrary (path);
+  SString missing = loadLibrary (path);
   hunspell = 0;
   functions.create = 0;
   functions.spell = 0;
@@ -87,10 +87,17 @@ SHunspellPattern::SHunspellPattern (const SString& _name,
   if (dynHandle)
   {
 #ifdef USE_WINAPI
+    /* Old exports are kept for compatibility with libhunspell-1.2.8.p1 */
     functions.create = (void* (*)(const char*, const char*)) GetProcAddress((HMODULE)dynHandle, "hunspell_initialize");
     functions.spell = (int (*)(void*, const char*)) GetProcAddress((HMODULE)dynHandle, "hunspell_spell");
     functions.destroy = (int (*)(void*)) GetProcAddress((HMODULE)dynHandle, "hunspell_uninitialize");
     functions.get_dic_encoding = (char* (*)(void*))  GetProcAddress((HMODULE)dynHandle, "hunspell_get_dic_encoding");
+    if (!(functions.create && functions.spell && functions.destroy && functions.get_dic_encoding)) {
+        functions.create = (void* (*)(const char*, const char*)) GetProcAddress((HMODULE)dynHandle, "Hunspell_create");
+        functions.spell = (int (*)(void*, const char*)) GetProcAddress((HMODULE)dynHandle, "Hunspell_spell");
+        functions.destroy = (int (*)(void*)) GetProcAddress((HMODULE)dynHandle, "Hunspell_destroy");
+        functions.get_dic_encoding = (char* (*)(void*))  GetProcAddress((HMODULE)dynHandle, "Hunspell_get_dic_encoding");
+    }
 #else
     functions.create = (void* (*)(const char*, const char*)) dlsym(dynHandle, "Hunspell_create");
     functions.spell = (int (*)(void*, const char*)) dlsym(dynHandle, "Hunspell_spell");
@@ -365,7 +372,7 @@ SHunspellPattern::isJapanese (SS_UCS4 chr) const
   // 
   if (chr >= 0x31c0 && chr <=0x9fff) return true;
 
-  // Japanes fullwidth alphabet
+  // Japanese fullwidth alphabet
   if (chr >= 0xff21 && chr <=0xff5a) return false;
 
   // Halfwidth and Fullwidth Forms
