@@ -1,7 +1,7 @@
 /** 
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 1997-2006  Gaspar Sinai <gaspar@yudit.org>  
+ *  GNU Copyright (C) 1997-2023  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -18,6 +18,7 @@
  */
 
 #include <swidget/SCaret.h>
+#include "stoolkit/STypes.h" 
 
 /**
  * Create a caret.
@@ -131,7 +132,7 @@ SCaret::redraw (SCanvas* canvas, int x, int y, unsigned int width, unsigned int
   //bool lr = (direction == SS_DR_L 
   //     || direction == SS_DR_LE || direction == SS_DR_LO);
   /* This is the key for the drawing engine */
-  sprintf (a, "caret=%u %u %c;", 
+  snprintf (a, 64, "caret=%u %u %c;", 
          getSize().width, getSize().height, 
         (char)direction);
 
