@@ -1,7 +1,7 @@
 /** 
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 1997-2006  Gaspar Sinai <gaspar@yudit.org>  
+ *  GNU Copyright (C) 1997-2023  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -360,7 +360,7 @@ key==SWindowListener::Key_Shift_L)
    if (ctrl|meta)
     {
       if (editorIF->isSelecting()) editorIF->endSelect();
-      editorIF->clipSelect();
+      editorIF->clipSelect(false);
       return; /* Automatic copy */
     }
   /* This is used with accelerator anyway - this is never called here */
@@ -479,7 +479,7 @@ key==SWindowListener::Key_Shift_L)
     if (ctrl|meta)
     {
       if (editorIF->isSelecting()) editorIF->endSelect();
-      pasteText (b);
+      pasteText (b, false);
       return;
     }
   case SWindowListener::Key_X:
@@ -488,7 +488,7 @@ key==SWindowListener::Key_Shift_L)
     {
       /* switch - delete */
       std = SUndo::SS_EraseSelect;
-      editorIF->clipSelect();
+      editorIF->clipSelect(false);
       STextIndex mba = editorIF->getSelectedIndex(true);
       erasedString = editorIF->eraseSelectedText();
       if (cleared) return;
@@ -569,7 +569,7 @@ SEditor::buttonPressed (int button, const SCursorIndex& index)
   editorIF->deselectText();
   if (button == 1)
   {
-    pasteText(editorIF->getCursorIndex());
+    pasteText(editorIF->getCursorIndex(), true);
     return;
   }
   /*mousewheel*/
@@ -588,14 +588,14 @@ SEditor::buttonPressed (int button, const SCursorIndex& index)
  * paste a text ftom clip
  */
 void
-SEditor::pasteText (const SCursorIndex& index)
+SEditor::pasteText (const SCursorIndex& index, bool isPrimary)
 {
   if (editorIF == 0) return;
   editorIF->deselectText();
   editorIF->setCursorIndex(index);
   SCursorIndex b = index;
   /* stop input methods */
-  editorIF->insertClipboardText();
+  editorIF->insertClipboardText(isPrimary);
   SCursorIndex ba = editorIF->getCursorIndex();
   SString s = editorIF->getText (b.getTextIndex(), ba.getTextIndex());
   SCursorIndex a = editorIF->getCursorIndex();
