@@ -1,7 +1,7 @@
 /** 
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 1997-2006  Gaspar Sinai <gaspar@yudit.org>  
+ *  GNU Copyright (C) 1997-2023  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -113,6 +113,7 @@ SFileDialog::SFileDialog (void)
   add (fileTypeButton);
   fileTypeLabel = new SLabel ("[]");
   add (fileTypeLabel);
+
   textDialog = new STextDialog();
   textDialog->setModal (this);
   recalc();
@@ -244,6 +245,9 @@ void
 SFileDialog::recalc ()
 {
   unsigned int bw = 1;
+  // MARGIN 
+  int MGN = (int) (SAwt::getScale() * 5); 
+
   if (yesButton->getPreferredSize().width > bw )
   {
      bw = yesButton->getPreferredSize().width;
@@ -261,85 +265,105 @@ SFileDialog::recalc ()
      bw = cancelButton->getPreferredSize().width;
   }
   unsigned int bh= cancelButton->getPreferredSize().height;
-  int gap = (300-((int)bw * 4 + 10))/3;
-  int currentx= 5;
+  int currentx= MGN;
 
+  SDimension hms = homeButton->getPreferredSize();
+  SDimension hs = hiddenButton->getPreferredSize();
+  SDimension bs = fileTypeButton->getPreferredSize();
+  SDimension fs = filterTitleLabel->getPreferredSize();
+  int filterWidth = (int) (SAwt::getScale() * 80.0); //* scale.
+
+  unsigned int bestWidthBottom = bw * 4 + MGN*MGN + 3 *MGN;
+  /* top */
+  unsigned int bestWidthTop = MGN + hms.width + MGN + hs.width 
+          + MGN +fs.width + filterWidth + bs.width + MGN + 2*MGN;
+  unsigned int bestWidth = (bestWidthTop > bestWidthBottom) ?
+        bestWidthTop : bestWidthBottom;
+  preferredSize = SDimension (bestWidth+2*MGN,  (bestWidth+2*MGN)*3/5);
+
+  unsigned int PW = preferredSize.width * 2;
+  unsigned int PH = preferredSize.height * 2;
+
+  int gap = (PW - MGN - MGN - ((int)bw * 4))/3;
+
+//fprintf (stderr, "currentX[0] - %d\n", currentx);
   yesButton->setLayout (
     SLayout (
-      SLocation (currentx, 195-bh),
-      SLocation (currentx+(int)bw, 195),
+      SLocation (currentx, PH-MGN-bh),
+      SLocation (currentx+(int)bw, PH-MGN),
       SLocation (0, 100),
       SLocation (0, 100)
     )
   );
   currentx = currentx + (int) bw + gap;
+//fprintf (stderr, "currentX[1] - %d\n", currentx);
   cdButton->setLayout (
     SLayout (
-      SLocation (currentx, 195-bh),
-      SLocation (currentx+(int)bw, 195),
+      SLocation (currentx, PH-MGN-bh),
+      SLocation (currentx+(int)bw, PH-MGN),
       SLocation (33, 100),
       SLocation (33, 100)
     )
   );
   currentx = currentx + (int) bw + gap;
+//fprintf (stderr, "currentX[4] - %d\n", currentx);
   createButton->setLayout (
     SLayout (
-      SLocation (currentx, 195-bh),
-      SLocation (currentx+(int)bw, 195),
+      SLocation (currentx, PH-MGN-bh),
+      SLocation (currentx+(int)bw, PH-MGN),
       SLocation (66, 100),
       SLocation (66, 100)
     )
   );
   currentx = currentx + (int) bw + gap;
+//fprintf (stderr, "currentX[4] - %d\n", currentx);
   cancelButton->setLayout (
     SLayout (
-      SLocation (currentx, 195-bh),
-      SLocation (currentx+(int)bw, 195),
-      SLocation (100, 100),
+      SLocation (currentx, PH-MGN-bh),
+      SLocation (currentx+(int)bw, PH-MGN),
+      SLocation (99, 100),
       SLocation (100, 100)
     )
   );
-  SDimension hms = homeButton->getPreferredSize();
   homeButton->setLayout (
     SLayout (
-      SLocation (5, 5),
-      SLocation ((int)hms.width+5, (int)hms.height+5),
+      SLocation (MGN, MGN),
+      SLocation ((int)hms.width+MGN, (int)hms.height+MGN),
       SLocation (0, 0),
       SLocation (0, 0)
     )
   );
-  SDimension hs = hiddenButton->getPreferredSize();
   hiddenButton->setLayout (
     SLayout (
-      SLocation ((int)hms.width+10, 5),
-      SLocation ((int)hms.width + hs.width+10, (int)hs.height+5),
+      SLocation ((int)hms.width+MGN+MGN, MGN),
+      SLocation ((int)hms.width + hs.width+10, (int)hs.height+MGN),
       SLocation (0, 0),
       SLocation (0, 0)
     )
   );
-  SDimension bs = fileTypeButton->getPreferredSize();
   fileTypeButton->setLayout (
     SLayout (
-      SLocation (295-(int)bs.width, 5),
-      SLocation (295, (int)bs.height+5),
+      SLocation (PW-MGN-(int)bs.width, MGN),
+      SLocation (PW-MGN, (int)bs.height+MGN),
       SLocation (100, 0),
       SLocation (100, 0)
     )
   );
-  SDimension fs = filterTitleLabel->getPreferredSize();
+  int PX12 = (int) (SAwt::getScale() * 12); //* scale.
   filterTitleLabel->setLayout (
     SLayout (
-      SLocation ((int)hs.width+(int)hms.width+12, 5),
-      SLocation ((int) fs.width + (int)hs.width+(int)hms.width+12, (int) fs.height+5),
+      SLocation ((int)hs.width+(int)hms.width+PX12, MGN),
+      SLocation ((int) fs.width + (int)hs.width+(int)hms.width+PX12, (int) fs.height+MGN),
       SLocation (0, 0),
       SLocation (0, 0)
     )
   );
+
   filterTextEdit->setLayout (
     SLayout (
-      SLocation ((int) fs.width + (int)hs.width+(int)hms.width+12, 5),
-      SLocation ((int) fs.width + 80 + (int)hs.width+(int)hms.width+12, 
-             (int) fs.height+5),
+      SLocation ((int) fs.width + (int)hs.width+(int)hms.width+PX12, MGN),
+      SLocation ((int) fs.width + filterWidth + (int)hs.width+(int)hms.width+PX12, 
+             (int) fs.height+MGN),
       SLocation (0, 0),
       SLocation (0, 0)
     )
@@ -354,8 +378,8 @@ SFileDialog::recalc ()
 
   folderTitleLabel->setLayout (
      SLayout (
-        SLocation (5, 200-(int)bh-(int)tfh -(int) dph - 10),
-        SLocation (5 + dpw, 200-(int)bh -(int)tfh - 10),
+        SLocation (MGN, PH-(int)bh-(int)tfh -(int) dph - MGN-MGN),
+        SLocation (MGN + dpw, PH-(int)bh -(int)tfh - MGN-MGN),
         SLocation (0, 100),
         SLocation (0, 100)
      )
@@ -363,8 +387,8 @@ SFileDialog::recalc ()
 
   folderNameLabel->setLayout (
      SLayout (
-        SLocation (5+dpw, 200-(int)bh-(int)tfh -(int) dph - 10),
-        SLocation (295, 200-(int)bh -(int)tfh - 10),
+        SLocation (MGN+dpw, PH-(int)bh-(int)tfh -(int) dph - MGN-MGN),
+        SLocation (PW-MGN, PH-(int)bh -(int)tfh - MGN-MGN),
         SLocation (0, 100),
         SLocation (100, 100)
      )
@@ -372,35 +396,36 @@ SFileDialog::recalc ()
 
   fileTitleLabel->setLayout (
      SLayout (
-        SLocation (5, 200-(int)bh-(int)tfh - 10),
-        SLocation (5 + phw, 200-(int)bh - 10),
+        SLocation (MGN, PH-(int)bh-(int)tfh - MGN-MGN),
+        SLocation (MGN + phw, PH-(int)bh - MGN-MGN),
         SLocation (0, 100),
         SLocation (0, 100)
      )
   );
+  int fileTypeWidth = (int) (SAwt::getScale() * 115.0); //* scale.
 
   fileTextEdit->setLayout (
      SLayout (
-        SLocation (5+phw, 200-(int)bh-(int)tfh - 10),
-        SLocation (180, 200-(int)bh - 10),
+        SLocation (MGN+phw, PH-(int)bh-(int)tfh - MGN-MGN),
+        SLocation (PW-MGN-fileTypeWidth, PH-(int)bh - MGN-MGN),
         SLocation (0, 100),
         SLocation (100, 100)
      )
   );
   fileTypeLabel->setLayout (
      SLayout (
-        SLocation (180, 200-(int)bh-(int)tfh - 10),
-        SLocation (295, 200-(int)bh - 10),
+        SLocation (PW-MGN-fileTypeWidth, PH-(int)bh-(int)tfh - MGN-MGN),
+        SLocation (PW-MGN, PH-(int)bh - MGN-MGN),
         SLocation (100, 100),
         SLocation (100, 100)
      )
   );
 
-  unsigned int incr = (300-10)/3;
+  unsigned int incr = (PW-MGN-MGN)/3;
   folderListBox->setLayout (
     SLayout (
-      SLocation (5, (int)hs.height+5+5),
-      SLocation (5+incr-2, 200-(int)bh-(int)tfh -(int) dph - 10),
+      SLocation (MGN, (int)hs.height+MGN+MGN),
+      SLocation (MGN+incr-2, PH-(int)bh-(int)tfh -(int) dph - MGN-MGN),
       SLocation (0, 0),
       SLocation (33, 100)
     )
@@ -408,8 +433,8 @@ SFileDialog::recalc ()
 
   fileListBox->setLayout (
     SLayout (
-      SLocation (5+incr+1, (int)hs.height+5+5),
-      SLocation (300-5-incr-1, 200-(int)bh-(int)tfh-(int) dph  - 10),
+      SLocation (MGN+incr+1, (int)hs.height+MGN+MGN),
+      SLocation (PW-MGN-incr-1, PH-(int)bh-(int)tfh-(int) dph  - MGN-MGN),
       SLocation (33, 0),
       SLocation (66, 100)
     )
@@ -417,26 +442,19 @@ SFileDialog::recalc ()
 
   fileTypeListBox->setLayout (
     SLayout (
-      SLocation (300-5-incr+2, (int)hs.height+5+5),
-      SLocation (300-5, 200-(int)bh-(int)tfh-(int) dph  - 10),
+      SLocation (PW-MGN-incr+2, (int)hs.height+MGN+MGN),
+      SLocation (PW-MGN, PH-(int)bh-(int)tfh-(int) dph  - MGN-MGN),
       SLocation (66, 0),
       SLocation (100, 100)
     )
   );
-  unsigned int bestWidthBottom = bw * 4 + 10 + 15 + 20;
-  /* top */
-  unsigned int bestWidthTop = 5 + hms.width + 5 + hs.width 
-          + 5 +fs.width + 80 + bs.width + 5 + 10;
-  unsigned int bestWidth = (bestWidthTop>bestWidthBottom) ?
-        bestWidthTop : bestWidthBottom;
 
-  preferredSize = SDimension (bestWidth+10,  300);
-
-  /* This was out layout... */
-  forceLayout (SLayout (SDimension (300,200)));
+  /* This was our layout... */
+  forceLayout (SLayout (SDimension (PW,PH)));
 
   /* But this one is the one that works */
-  setLayout (SLayout (preferredSize));
+  //orig setLayout (SLayout (preferredSize));
+  setLayout (SLayout (SDimension (PW,PH)));
 
   setMinimumSize (preferredSize);
 
@@ -676,7 +694,7 @@ SFileDialog::filterChanged ()
 }
 
 /**
- * set icon accoring to showHidden
+ * set icon accodring to showHidden
  */
 void
 SFileDialog::setHidden()
@@ -797,6 +815,9 @@ SFileDialog::setFont (const SString& font, double fontSize)
   folderListBox->setFont (font, fontSize);
   fileListBox->setFont (font, fontSize);
   fileTypeListBox->setFont (font, fontSize);
+
+  fileTextEdit->setFont (font, fontSize);
+  filterTextEdit->setFont (font, fontSize);
   recalc();
 }
 
@@ -819,6 +840,9 @@ SFileDialog::setFontSize (double fontSize)
   folderListBox->setFontSize (fontSize);
   fileListBox->setFontSize (fontSize);
   fileTypeListBox->setFontSize (fontSize);
+
+  fileTextEdit->setFontSize (fontSize);
+  filterTextEdit->setFontSize (fontSize);
   recalc();
 }
 /**
