@@ -1,7 +1,7 @@
 /* 
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 1997-2006  Gaspar Sinai <gaspar@yudit.org>  
+ *  GNU Copyright (C) 1997-2023  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -204,7 +204,7 @@ STextEdit::removeTextEditLS (STextEditLS* ls)
 /**
  * Set the overall background. This will effectively be the border's 
  * background.
- * @param bg is the backgroud.
+ * @param bg is the background.
  */
 void
 STextEdit::setBackground (const SColor& bg)
@@ -214,7 +214,7 @@ STextEdit::setBackground (const SColor& bg)
 
 /**
  * Set the background of the text itself.
- * @param bg is the backgroud.
+ * @param bg is the background.
  */
 void
 STextEdit::setTextBackground (const SColor& bg)
@@ -1088,9 +1088,9 @@ STextEdit::setInputMethod (const SString& _im)
  * Get text from clipboard and put it here.
  */
 void
-STextEdit::insertClipboardText ()
+STextEdit::insertClipboardText (bool isPrimary)
 {
-  SString str = window->getClipUTF8();
+  SString str = window->getClipUTF8(isPrimary);
   if (str.size()==0)
   {
     return;
@@ -1136,12 +1136,12 @@ STextEdit::insertClipboardText ()
 void
 STextEdit::autoClipSelection ()
 {
-  if (autoClipboard) clipSelect();
+  if (autoClipboard) clipSelect(true);
   
 }
 
 void
-STextEdit::clipSelect ()
+STextEdit::clipSelect (bool isPrimary)
 {
   if (textView.isHideText()) return;
 
@@ -1150,7 +1150,7 @@ STextEdit::clipSelect ()
     return;
   }
   SString str = textView.textData.getText (startSelection, endSelection);
-  window->putClipUTF8 (str);
+  window->putClipUTF8 (str, isPrimary);
 }
 
 // Return an empty string if there is no selected text, or
@@ -1169,7 +1169,7 @@ STextEdit::getSelectedText ()
 /**
  * Dirty text may conta8ing newline characters and the line. 
  * This routine is calling insertText for multiline widget.
- * For a signle line widget it is chopping the text at newline
+ * For a single line widget it is chopping the text at newline
  * and calls the textEntered callback.
  *
  * @param str is an utf8 string that may contain line-breaks as well.
@@ -2615,7 +2615,7 @@ STextEdit::isEmbedStateLR() const
 /**
  * Change the direction of the selected text.
  * If no text is selected insert new direction.
- * @return true if the direction of cursor succeded, false
+ * @return true if the direction of cursor succeeded, false
  *  if only text embedding has been changed.
  */
 bool
@@ -2794,7 +2794,7 @@ STextEdit::changeDirection (SS_DR_Dir dir)
 
 
 /**
- * Move the caret to preceeding getTextIndex position.
+ * Move the caret to preceding getTextIndex position.
  * @param updateColumn is true if column needs updating (dafult)
  */
 void
@@ -2892,7 +2892,7 @@ STextEdit::internalMoveCaret (const SCursorIndex& realIndex)
 
   /* check if previous embed state is different */
 
-  /* if this embedding state is different from sorrounding embedding
+  /* if this embedding state is different from surrounding embedding
      state change cursor shape tu current embedding state. */
   unsigned int line = realIndex.textIndex.line;
   unsigned int max = textView.textData.size(line);
@@ -2994,7 +2994,7 @@ STextEdit::getProperties ()
 
   SColor bg = textView.getBackground();
   SColor fg = textView.getForeground (true);
-  sprintf (arr, "%lu,%lu", 
+  snprintf (arr, 64, "%lu,%lu", 
       (unsigned long)bg.getValue(), (unsigned long)fg.getValue());
   if (ximProperties.get ("InputClientColor"))
   {
@@ -3015,7 +3015,7 @@ STextEdit::getProperties ()
   }
 
   SLocation caretL = caret.getLocation();
-  sprintf (arr, "%d,%d", caretL.x, caretL.y + textView.lineAscent);
+  snprintf (arr, 64, "%d,%d", caretL.x, caretL.y + textView.lineAscent);
   p.put ("InputSpot", arr);
 
   /* add required and optional parameters if any */
@@ -3023,7 +3023,7 @@ STextEdit::getProperties ()
   {
     SWindow* w = getComponentWindow();
     unsigned long wid = w->getWindowID();
-    sprintf (arr, "%lu", wid);
+    snprintf (arr, 64, "%lu", wid);
     p.put ("InputWindow", arr);
 
     SLocation vl = textView.getLocation();
@@ -3031,21 +3031,21 @@ STextEdit::getProperties ()
     SDimension vd = textView.getSize();
     vd.width = vd.width + (unsigned int) 2 * SS_LEFT_MARGIN;
     vl.y = vl.y + (int) vd.height;
-    sprintf (arr, "%d,%d", vl.x, vl.y); 
+    snprintf (arr, 64, "%d,%d", vl.x, vl.y); 
     p.put ("InputStatusLocation", arr);
     unsigned int statusw = 2 * statusHeight;
     if (statusw < 48) statusw = 48;
 
-    sprintf (arr, "%u,%u", statusw, statusHeight);
+    snprintf (arr, 64, "%u,%u", statusw, statusHeight);
     p.put ("InputStatusSize", arr);
 
     vl.x = (int) statusw;
-    sprintf (arr, "%d,%d", vl.x, vl.y); 
+    snprintf (arr, 64, "%d,%d", vl.x, vl.y); 
     p.put ("InputClientLocation", arr);
 
     vd.width = vd.width + (unsigned int) 2 * SS_LEFT_MARGIN;
     if (vd.width > statusw) vd.width = vd.width - statusw;
-    sprintf (arr, "%u,%u", vd.width, statusHeight);
+    snprintf (arr, 64, "%u,%u", vd.width, statusHeight);
     p.put ("InputClientSize", arr);
    
   } 
@@ -3057,10 +3057,10 @@ STextEdit::getProperties ()
     SDimension vd = textView.getSize();
     vd.width = vd.width + (unsigned int) 2 * SS_LEFT_MARGIN;
     vl.y = vl.y + (int) vd.height;
-    sprintf (arr, "%d,%d", vl.x, vl.y); 
+    snprintf (arr, 64, "%d,%d", vl.x, vl.y); 
     p.put ("InputStatusLocation", arr);
 
-    sprintf (arr, "%u,%u", vd.width, statusHeight);
+    snprintf (arr, 64, "%u,%u", vd.width, statusHeight);
     p.put ("InputStatusSize", arr);
   } 
 
@@ -3120,7 +3120,7 @@ STextEdit::preEditInsertMarkedText(const SString& utf8, SStyle style=Style_Defau
         return;
     }
     insertPreEditText (utf8, style);
-    // FIXME figure out where the first caracter is.
+    // FIXME figure out where the first character is.
     preEditEndIndex = getCursorIndex();
     preEditSize++;
     return;
