@@ -1,7 +1,7 @@
 /* 
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 1997-2006  Gaspar Sinai <gaspar@yudit.org>  
+ *  GNU Copyright (C) 1997-2023  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -17,6 +17,7 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#include "swindow/SAwt.h"
 #include "swidget/STextView.h"
 #include "stoolkit/SCluster.h"
 
@@ -34,6 +35,8 @@
 #include <time.h>
 #endif
 
+#include "swindow/SAwt.h"
+
 
 static struct timeval thatTime;
 
@@ -63,14 +66,22 @@ timerStop()
 
 static unsigned int sane_index (const SV_UINT& array, unsigned int index);
 
+
+static double scaleFontSize (bool scaleable, double _fontSize) {
+    if (!scaleable) return _fontSize;
+    double ret = SAwt::getScale() * _fontSize;   
+    return ret;
+}
+
 /**
  * The text data is mine. I'll delete it.
  */
-STextView::STextView (void)
+STextView::STextView (bool _scaleable)
   : lrpen (SColor (0.0, 0.0, 0.0, 1.0)),
   rlpen (SColor (0.0, 0.0, 1.0, 1.0)),
   underlineColor("red")
 {
+  scaleable = _scaleable;
   isHidingText = false;
   printerPageSize = 0;
   highlightMode = "";
@@ -78,7 +89,9 @@ STextView::STextView (void)
   isEditable = false;
   clipx = clipy = 0;
   clipw = cliph = 0;
+  // the default fonts size
   fontSize = 16.0;
+//fprintf (stderr, "Scaled font size=%g\n", fontSize);
   lineend = true;
   multiline = true;
   syntax.setTextData (&textData);
@@ -91,11 +104,12 @@ STextView::STextView (void)
  * Make a text-view from external text
  * @param utf8 is the utf8 ancoded text
  */
-STextView::STextView (const SString& utf8)
+STextView::STextView (const SString& utf8, bool _scaleable)
   : textData (utf8), lrpen (SColor (0.0, 0.0, 0.0, 1.0)),
   rlpen (SColor (0.0, 0.0, 1.0, 1.0)),
   underlineColor("red")
 {
+  scaleable = _scaleable;
   isHidingText = false;
   printerPageSize = 0;
   highlightMode = "";
@@ -103,7 +117,9 @@ STextView::STextView (const SString& utf8)
   isEditable = false;
   clipx = clipy = 0;
   clipw = cliph = 0;
+  // the default fonts size
   fontSize = 16.0;
+//fprintf (stderr, "Scaled font size=%g\n", fontSize);
   lineend = true;
   multiline = true;
   wrapAndPosition ();
@@ -126,7 +142,7 @@ STextView::setText(const SString& text)
   textData.fireEvent();
   textData.insert(text);
  
-  /* HACK FOR LABELS - they neeed to know their exact size */
+  /* HACK FOR LABELS - they need to know their exact size */
   for (unsigned int i=0; i<textData.size(); i++)
   {
     textData.setVisible(i);
@@ -157,7 +173,7 @@ STextView::setClippingArea (int _x, int _y,
 void
 STextView::setFont (const SString& _font, double _fontSize)
 {
-  if (_fontSize > 0.0) fontSize = _fontSize;
+  if (_fontSize > 0.0) fontSize = scaleFontSize (scaleable, _fontSize);
   font = SFont(_font, fontSize);
   setPen ();
   setReordered ();
@@ -173,9 +189,11 @@ STextView::setFont (const SString& _font, double _fontSize)
  * @param size is the size of the font.
  */
 void
-STextView::setFontSize (double newsize)
+STextView::setFontSize (double _fontSize)
 {
-  font.setSize(newsize);
+  if (_fontSize > 0.0) fontSize = scaleFontSize (scaleable, _fontSize);
+  font.setSize(fontSize);
+  
   setPen ();
   setReordered ();
 
@@ -447,7 +465,7 @@ STextView::getTextLocation (const STextIndex& textIndex, bool before)
       si = 0;
       yoffset++;
     }
-    else /* Streched beyond the last one. */
+    else /* Stretched beyond the last one. */
     { 
       SV_UINT v = posAfter[line];
       unsigned int max = 0;
@@ -642,7 +660,7 @@ STextView::drawParagraph (SCanvas* c, bool islr, unsigned int line,
 }
 
 /**
- * Set Syntax Hilight Mode
+ * Set Syntax Highlight Mode
  */
 void
 STextView::setSyntax (const SString& hlm)
@@ -660,7 +678,7 @@ STextView::setSyntax (const SString& hlm)
 }
 
 /**
- * Get Syntax Hilight Mode
+ * Get Syntax Highlight Mode
  */
 const SString&
 STextView::getSyntax () const
@@ -715,7 +733,7 @@ STextView::getWordWrap () const
  * Syntax Highlighting  system.
  * Added by Maarten van Gompel <proycon@anaproy.homeip.net>
  * Note:  this is a dumb system and merely colors single characters.
- * For error highlighting, set forground to NONE and error to true.
+ * For error highlighting, set foreground to NONE and error to true.
  */
 void
 STextView::syntaxHighlight(STextIndex index, SPen* pen, bool *isError)
@@ -796,7 +814,7 @@ STextView::syntaxHighlight(STextIndex index, SPen* pen, bool *isError)
 }
 
 /**
- * Draw one signle glyph on the screen.
+ * Draw one single glyph on the screen.
  * @param c is the canvas to draw to
  * @param l is the location of the glyph.
  * @return the length of the text drawn
@@ -893,21 +911,28 @@ STextView::drawGlyph (SCanvas* c,
     unsigned int base =  (lineAscent + h >= lineHeight) ?
          lineHeight - 1 : lineAscent + h;
     int screenBase = l.y + (int) base;
+    int raster = SAwt::getRasterScale();
+
     SBinVector<int> x;
     SBinVector<int> y;
     for (unsigned i=0; i<w; i++)
     {
-      x.append (l.x + i);
-      int state = ((l.x+i+clipx) % 6);
+      int state = (((l.x+i+clipx)/raster) % 6);
+      int ybase = screenBase;
       switch (state)
       {
-      case 0: y.append (screenBase); break;
-      case 1: y.append (screenBase-1); break;
-      case 2: y.append (screenBase-1); break;
-      case 3: y.append (screenBase); break;
-      case 4: y.append (screenBase+1); break;
-      case 5: y.append (screenBase+1); break;
-      default: y.append (screenBase);
+      case 0: ybase = screenBase; break;
+      case 1: ybase = screenBase-raster; break;
+      case 2: ybase = screenBase-raster; break;
+      case 3: ybase = screenBase; break;
+      case 4: ybase = screenBase+raster; break;
+      case 5: ybase = screenBase+raster; break;
+      default: ybase = screenBase;
+      }
+      for (int j=0; j<raster; j++) 
+      {
+         x.append (l.x + i);
+         y.append (ybase+j); 
       }
     }
     if (x.size() > 0)
@@ -1270,7 +1295,7 @@ STextView::textChangedInternal (void* src, const STextDataEvent& event)
 void 
 STextView::setReordered()
 {
-  /* HACK FOR LABELS - they neeed to know their exact size */
+  /* HACK FOR LABELS - they need to know their exact size */
   if (!isEditable)
   {
     for (unsigned int i=0; i<textData.size(); i++)
