@@ -1,7 +1,7 @@
 /** 
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 1997-2006  Gaspar Sinai <gaspar@yudit.org>  
+ *  GNU Copyright (C) 1997-2023  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -26,6 +26,90 @@
 #include "stoolkit/SExcept.h"
 #include "stoolkit/SCluster.h"
 
+const SString&
+SFont::getDefaultFontList ()
+{
+    static SString ret;
+    if (ret.size() > 0) {
+        return ret;
+    }
+    ret = SString (
+        /* 16: yudit.hex has also Hosszu Gabors PUA old hungarian. */
+        "yudit.hex,arabforms.hex,syriacforms.hex,unifont.hex,"
+        /* 18: */
+        "markus9x18.bdf,markus18x18ja.bdf,"
+        // recommended Noto fonts
+        "NotoSansGujarati-Regular.ttf:gujr,"
+        "NotoSansDevanagari-Regular.ttf:deva,"
+        "NotoSansBengali-Regular.ttf:beng,"
+        "NotoSansGurmukhi-Regular.ttf:guru,"
+        "NotoSansOriya-Regular.ttf:orya,"
+        "NotoSansTamil-Regular.ttf:taml,"
+        "NotoSansTelugu-Regular.ttf:telu,"
+        "NotoSansMalayalam-Regular.ttf:mlym,"
+        "NotoSansSinhala-Regular.ttf:sinh,"
+        "NotoSansThai-Regular.ttf:thai,"
+        "NotoSansLao-Regular.ttf:lao,"
+        "NotoSansKannada-Regular.ttf:knda,"
+/* This screws up line height. removed. add it as tibt.tttf if you need it.
+        "NotoSansTibetan-Regular.ttf:tibt," 
+*/
+        // I kept this excellent font 
+        "raghu.ttf:deva,"
+
+        "Arial.ttf,"
+        "NotoSans-Regular.ttf,"
+
+        // Japanese
+        "VLGothicReqular.ttf,"
+        // Hangul 
+        "gulim.ttf,"
+        "ogulim.ttf:mslvt,"
+//        "SansSerif.ttf,"
+        "NotoSansEthiopic-Regular.ttf,"
+        "NotoSansArabic-Regular.ttf,"
+        "NotoSansSyriac-Regular.ttf,"
+        "NotoSansTagalog-Regular.ttf,"
+        "NotoSansRunic-Regular.ttf,"
+        "NotoSansOldItalic-Regular.ttf,"
+        "NotoSansSymbols-Regular.ttf,"
+        "NotoSansSymbols2-Regular.ttf,"
+        "NotoSansCanadianAboriginal-Regular.ttf,"
+        "NotoSansCherokee-Regular.ttf,"
+        "NotoSansGlagolitic-Regular.ttf,"
+        "NotoSansHanunoo-Regular.ttf,"
+        "NotoSansShavian-Regular.ttf,"
+        // emoji, Ubuntu installs it by font-name
+        "EmojiOne_Color.otf:emoji,"
+        // emoji, Mac installs it by file-name
+        "EmojiOneColor.otf:emoji,"
+
+        "TwitterColorEmoji-SVGinOT.ttf:emoji,"
+        /* ubuntu system usually has these */
+        "Ubuntu-R.ttf,"
+        "fonts-japanese-gothic.ttf,"
+        "ipag.ttf,"
+        /* Macintosh */
+        "SFNS.ttf,"
+        "SFArabic.ttf,"
+        // Last resorts
+        "Arialuni.ttf,"
+        "Arial Unicode.ttf,"
+        // Font vendor: Agfa Monotype Corporation
+        "arial-unicode-ms.ttf,"
+        // From Yudit 
+        "OldHungarian_Full.ttf:unicode:RL,"
+        //"FreeSans.ttf,"
+        "unifont.ttf,"
+        // From Yudit 
+        "yudit.ttf,"
+        /* X11 */
+        "-*-*-medium-r-normal--16-*-*-*-c-*-iso8859-1,"
+        "-*-*-*-*-*--16-*-*-*-c-*-iso8859-1,"
+    );
+    return ret;
+}
+
 static SFontFB fallbackFont;
 typedef SHashtable<SFontImplVector> SFontHashtable;
 static SFontHashtable* fontHashtable=0;
@@ -44,6 +128,7 @@ SFont::SFont (const SString _name) : name(_name), xlfd (SD_XLFD_ANY)
  if (fontHashtable->get (name) != 0) fontVector = (*fontHashtable)[name];
  setSize (16.0);
 }
+
 /**
  * Create a font. The font can have many faces. 
  * @param _name is the logical name of this font.
@@ -55,6 +140,7 @@ SFont::SFont (const SString _name, double _size) : name(_name), xlfd (SD_XLFD_AN
  setSize (_size);
 }
 
+
 /**
  * get a point 16 font called _default
  */
@@ -63,31 +149,8 @@ SFont::SFont (void) : name("default"), xlfd (SD_XLFD_ANY)
   static bool _defaultSet=false;
   if (_defaultSet==false)
   {
-    SStringVector m(
-        /* yudit.hex has ohungarian.hex and part of unifont.hex */
-        // Indic range is not necessary free. But they are the best
-        "yudit.hex,arabforms.hex,syriacforms.hex,unifont.hex,"
-        "markus9x18.bdf,markus18x18ja.bdf,"
-        "-*-*-medium-r-normal--16-*-*-*-c-*-iso8859-1,"
-        "-*-*-*-*-*--16-*-*-*-c-*-iso8859-1,"
-        "THOOLIUC.TTF:mlym,"
-        "MuktiNarrow.ttf:beng,"
-        "ani.ttf:beng,"
-        "pothana2000.ttf:telu,"
-        "TCRCYoutsoUnicode.ttf:tibt,"
-        "raghu.ttf:deva,"
-        "mangal.ttf:deva,tunga.ttf:knda,code2000.ttf:taml,"
-        "raavi.ttf:guru,shruti.ttf:gujr,"
-        "arialuni.ttf,cyberbit.ttf,"
-        "code2000.ttf,code2001.ttf:unicode:RL,arial.ttf,"
-        //"rovasSMP.ttf," // Hosszu Gabor
-        //"oldhunSMP.ttf," // Michael Everson
-        "OldHungarian.ttf,"
-        "OldHungarian_Full.ttf,"
-        "yudit.ttf,"
-        "EmojiOneColor.otf:emoji,"
-        "TwitterColorEmoji-SVGinOT.ttf:emoji"
-    );
+    const SString& fl = getDefaultFontList();
+    SStringVector m(fl);
     SFontImplVector list;
     for (unsigned int i=0; i<m.size(); i++)
     {
@@ -340,7 +403,10 @@ SFont::draw (SCanvas* canvas, const SPen& pen, const SS_Matrix2D& m,
     unsigned int i;
 
     /* use mirrored glyphs for Old Hungarian, Old Italic */
-    if (  (comp >= 0xEE00 && comp < 0xEE9F)
+    /* getCharClass ROVASIRAS CAPS / ROVASIRAS SMALL */
+    // SFontImpl::needSoftMirror, 
+    // getPUARovasType
+    if (  (comp >= 0xee00 && comp < 0xef3f)
         || (comp >= 0x10300 && comp <= 0x1032F)
         || (comp >= 0x10c00 && comp <= 0x10fff)
         || scriptcode == SD_ROVASIRAS || scriptcode == SD_PUA_ROVAS)
@@ -433,6 +499,7 @@ SFont::draw (SCanvas* canvas, const SPen& pen, const SS_Matrix2D& m,
     }
     else
     {
+//fprintf (stderr, "XXX ScriptCode = %x shapedcahr=%x\n", scriptcode, comp);
        /* Try all fonts on it */
       for (i=0; i<fontVector.size(); i++)
       {
