@@ -1,7 +1,7 @@
 /** 
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 1997-2006  Gaspar Sinai <gaspar@yudit.org>  
+ *  GNU Copyright (C) 1997-2023  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -146,6 +146,7 @@ SFontTTF::isOK()
   if (name.size()==0)
   {
     name = file.getName();
+//fprintf (stdout, "%*.*s %*.*s\n", SSARGS(name), SSARGS(fontencoding));
     if (fontencoding == "mslvt")
     {
       hardWire = SS_MSLVT;
@@ -449,6 +450,7 @@ SFontTTF::init ()
 }
 
 
+static int anonym = 1;
 /**
  * Process the name table
  */
@@ -522,8 +524,21 @@ SFontTTF::processName ()
 
   if (names.get (SS_TN_FONTNAME) == 0 || names[SS_TN_FONTNAME].size() == 0)
   {
-    getName (SS_TN_FONTNAME, names[SS_TN_FULLNAME].array(),
-         names[SS_TN_FULLNAME].size());
+        if (names.get(SS_TN_FULLNAME) == 0) 
+        {
+            char an[64];
+            snprintf (an, 63, "anonym%04d", anonym++);
+            getName (SS_TN_FONTNAME, an, strlen(an));
+#if 0
+            fprintf (stderr, "SFontTTF: Assigned %s for %*.*s\n", 
+                an, SSARGS(name));
+#endif
+        }
+        else 
+        {
+            getName (SS_TN_FONTNAME, names[SS_TN_FULLNAME].array(),
+                names[SS_TN_FULLNAME].size());
+        }
   }
   return true;
 }
@@ -532,7 +547,7 @@ SFontTTF::processName ()
  * put the string from str into names. 
  * @param id is SS_TN_ something.
  * @param str is the input string
- * @param len is the size of the stirng
+ * @param len is the size of the string
  */
 void
 SFontTTF::getName (long id, const char* str, int len)
@@ -1195,7 +1210,7 @@ SFontTTF::descent (const SS_Matrix2D& m)
 }
 
 /**
- * Return the calibrated avarage width
+ * Return the calibrated average width
  * @param m is the transformation matrix.
  */
 double
@@ -1868,7 +1883,7 @@ SFontTTF::findGlyphs (SS_UCS4 in, SV_GlyphIndex* out)
                 * is represented by 'chillaksharams'. These can appear at any
                 *  non-initial or final consonant location in a syllable. 
                 *
-                * - unfortunatelly it is very vague: 'scripts like Malayalam'
+                * - unfortunately it is very vague: 'scripts like Malayalam'
                 *    gaspar
                 */
                if (inlen > 1)
@@ -2100,7 +2115,7 @@ SFontTTF::findGlyphs (SS_UCS4 in, SV_GlyphIndex* out)
   
   bool okToProcess = true;
 
-  /* Set okToProcess accoriding to artifical encodings */
+  /* Set okToProcess according to artificial encodings */
   switch (hardWire)
   {
   case SS_MSLVT:
@@ -2188,7 +2203,7 @@ SFontTTF::findGlyphs (SS_UCS4 in, SV_GlyphIndex* out)
   {
     /* Try the decomposed one instead */
     if (hardWire==SS_MSLVT && 
-       /* chekc for Precomposed Korean or JAMO */
+       /* check for Precomposed Korean or JAMO */
       ((in>=0xac00 && in<0xd7a4) || getJamoClass (in) != SD_JAMO_X))
     {
       SS_UCS4 chars[3]; /* lvt */
@@ -2473,6 +2488,9 @@ SFontTTF::findGlyph (SS_UCS4 in, bool ownjoiners)
     break;
   default: break;
   }
+  // Safeguard against making everything emoji
+  // EmojiOne_Color even makes numbers emoji.
+  if (isEmoji && in<0x2122) return 0;
 
   TTF_CMAP* cmap_table = (TTF_CMAP*) tables[SS_TB_CMAP];
   int num_tables = ntohs(cmap_table->numberOfEncodingTables);
@@ -2848,7 +2866,7 @@ SFontTTF::drawGlyph (SCanvas* canvas, const SS_Matrix2D& matrix,
   {
     if (!broken)
     {
-      fprintf  (stderr, "SFontTTF: fixing unaligned %*.*s.\n", SSARGS(name));
+// TODO: dont need this fprintf  (stderr, "SFontTTF: fixing unaligned %*.*s.\n", SSARGS(name));
       broken = true;
     }
     kludge = new TTF_GLYF[len]; 
@@ -3048,7 +3066,7 @@ uint8 or int16 yCoordinates[]
     // bad font
     if (cpc < 0) {
         if (!broken) {
-            fprintf  (stderr, "Bad TTF countour count %*.*s.\n", SSARGS(name));
+            fprintf  (stderr, "Bad TTF contour count %*.*s.\n", SSARGS(name));
         }
         broken = true;
         continue;
