@@ -1,7 +1,7 @@
 /** 
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 1997-2006  Gaspar Sinai <gaspar@yudit.org>  
+ *  GNU Copyright (C) 1997-2023  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -16,6 +16,13 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
+
+// #########################################################################
+// This code is obsolete. 
+// It was developed for the 2.x.x series of Yudit in year 2000
+// and was replaced in 2023 (Yudit-3.1.0) with SRasterizer.
+// This class is only kept here to do performance comparison tests.
+// #########################################################################
  
 #define SS_MAKKA true
 #include "swindow/SGEngine.h"
@@ -78,6 +85,10 @@ SGEngine::SGEngine(bool windingrule)
 
 SGEngine::~SGEngine ()
 {
+    for (unsigned int i=0; i<images.size(); i++) {
+        delete images[i];
+    }
+    images.clear();
 }
 
 /** 
@@ -207,7 +218,7 @@ SGEngine::_lineto (double _x, double _y)
 
 /**
  * Cubic bezier curve. first two points are the control points 
- * last point is the enpoint. Statring point is on the stack.
+ * last point is the endpoint. Statring point is on the stack.
  */
 void
 SGEngine::_curveto (double _x0, double _y0, double _x1, double _y1, 
@@ -564,6 +575,10 @@ SGEngine::_fill(int _x, int _y, unsigned int _width, unsigned int _height)
 void
 SGEngine::_pushmatrix()
 {
+  if (matrix.size() > 10) {
+    fprintf (stderr, "pushpmatrix size-%u\n", matrix.size());
+    return;
+  }
   SS_Matrix2D m = matrix[matrix.size()-1];
   matrix.append (m);
 }
@@ -571,6 +586,7 @@ SGEngine::_pushmatrix()
 void
 SGEngine::_popmatrix()
 {
+//  fprintf (stderr, "Popmatrix size-%u\n", matrix.size());
   if (matrix.size())
   {
     matrix.truncate(matrix.size()-1);
@@ -616,7 +632,7 @@ SGEngine::_translate (double x, double y)
  */
 
 /**
- * This routine is called when no coordiante transform is needed
+ * This routine is called when no coordnate transform is needed
  */
 void
 SGEngine::curvetoInternal (double x0, double y0, double x1, 
@@ -687,7 +703,7 @@ SGEngine::curvetoInternal (double x0, double y0, double x1,
 
 
 /**
- * This routine is called when no coordiante transform is needed
+ * This routine is called when no coordnate transform is needed
  */
 void
 SGEngine::linetoInternal (double x, double y)
@@ -705,9 +721,9 @@ SGEngine::linetoInternal (double x, double y)
  * If winding is true use the non-zero winding rule rather that
  * intersect rule.
  * @param inter in the allocated  and initialisez intersection array
- * @param offsetx is the offset in inter buffer
+ * @param offsetx is the offset in internal buffer
  * @param offsety is the offset in scanline.
- * @parm hieght is the hieght of the inter buffer 
+ * @parm height is the height of the internal buffer 
  * @param xy is 0 if x is x y is y 1 vice versa.
  */
 void
@@ -727,9 +743,9 @@ SGEngine::scan (SS_InterSection** intersBuff,
 /**
  * Scan the curves into the buffer using nonzero winding rule.
  * @param inter in the allocated  and initialisez intersection array
- * @param offsetx is the offset in inter buffer
+ * @param offsetx is the offset in internal buffer
  * @param offsety is the offset in scanline.
- * @parm hieght is the hieght of the inter buffer 
+ * @parm height is the height of the internal buffer 
  * @param xy is 0 if x is x y is y 1 vice versa.
  */
 void
@@ -827,7 +843,7 @@ SGEngine::scanWinding (SS_InterSection** intersBuff,
          unsigned int pos = intersBuff[k]->appendSorted ((int)is);
          clockwise[k]->insert (pos, increment);
          /*
-          * This happend if we got inside/out it all wrong
+          * This happened if we got inside/out it all wrong
           * Mainly happens when we get a straight line first and we
           * think is is positive.
           */
@@ -866,9 +882,9 @@ SGEngine::scanWinding (SS_InterSection** intersBuff,
 /**
  * Scan the curves into the buffer using crosses rule
  * @param inter in the allocated  and initialisez intersection array
- * @param offsetx is the offset in inter buffer
+ * @param offsetx is the offset in internal buffer
  * @param offsety is the offset in scanline.
- * @parm hieght is the hieght of the inter buffer 
+ * @parm height is the height of the internal buffer 
  * @param xy is 0 if x is x y is y 1 vice versa.
  */
 void
@@ -956,7 +972,7 @@ SGEngine::scanCrosses (SS_InterSection** intersBuff,
          }
          intersBuff[k]->appendSorted ((int)is);
          /*
-          * This happend if we got inside/out it all wrong
+          * This happened if we got inside/out it all wrong
           * Mainly happens when we get a straight line first and we
           * think is is positive.
           */
@@ -1023,7 +1039,7 @@ SGEngine::render (SS_WORD32* image, SS_InterSection** intersBuff,
  * aftwerwards. 
  *
  * IMPORTANT: it is the creator of the id that actually
- * is responsible of distinguishing diferent penWidth values.
+ * is responsible of distinguishing different penWidth values.
  *
  * If the image with id is not in cache, it will return 0.
  * @param id is the unique id of the image.
@@ -1035,6 +1051,9 @@ SGEngine::beginImage (int x, int y, const SString& id)
 {
   primitive.clear();
   pathVector.clear();
+  for (unsigned int i=0; i<images.size(); i++) {
+        delete images[i];
+  }
   images.clear();
   newpathID = id;
   newpathX = x;
@@ -1273,6 +1292,8 @@ SGEngine::closepath()
 void
 SGEngine::pushmatrix()
 {
+  // need this because scale is not a primitive...
+  _pushmatrix();
   SGPrimitive p; p.pushmatrix();
   primitive.append (p);
 }
@@ -1280,6 +1301,8 @@ SGEngine::pushmatrix()
 void
 SGEngine::popmatrix()
 {
+  // need this because scale is not a primitive...
+  _popmatrix();
   SGPrimitive p; p.popmatrix();
   primitive.append (p);
 }
