@@ -1,7 +1,7 @@
 /** 
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 1997-2006  Gaspar Sinai <gaspar@yudit.org>  
+ *  GNU Copyright (C) 1997-2023  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -40,6 +40,7 @@
  */
 SPostscript::SPostscript (const SWriter& wr, SMedia m, SOrientation o)  : out (wr)
 {
+  isCacheOn = true;
   ticks = 0;
   timer = 0;
   isUgly = false;
@@ -332,7 +333,7 @@ SPostscript::close ()
   }
   SStringVector l("%%DocumentFonts: Times-Roman");
   char a[64];
-  sprintf (a, "%%%%Pages: %u", pages);
+  snprintf (a, 64, "%%%%Pages: %u", pages);
   l.append (a);
   l.append ("%%Trailer");
   l.append ("%%EOF");
@@ -362,14 +363,14 @@ SPostscript::newPage()
   {
   case LANDSCAPE:
  
-          sprintf (buff, "\n%%%%Page: %u %u\nsave\n90 rotate\n0 -%d.000 translate\n", pages, currentPage, height);
+          snprintf (buff, 128, "\n%%%%Page: %u %u\nsave\n90 rotate\n0 -%d.000 translate\n", pages, currentPage, height);
  
     break;
  
   case PORTRAIT:
   default:
  
-          sprintf (buff, "\n%%%%Page: %u %u\nsave\n", pages, currentPage);
+          snprintf (buff, 128, "\n%%%%Page: %u %u\nsave\n", pages, currentPage);
  
     break;
   }
@@ -404,7 +405,7 @@ SPostscript::_beginImage (double _x, double _y, const SString& _id)
    */
   int ix =  (int) (1000.0 * _x);
   int iy =  (int) (1000.0 * ((double)height-_y));
-  sprintf (ins, "gsave %s%d.%03d %s%d.%03d translate ", 
+  snprintf (ins, 128, "gsave %s%d.%03d %s%d.%03d translate ", 
      SG(ix), ix/1000, SM (ix), SG(iy), iy/1000, SM(iy));
 
   SString all;
@@ -426,7 +427,7 @@ SPostscript::cacheOn (bool on)
   if (on) return isCacheOn;
 
   char p[64];
-  sprintf (p, "%u", pages);
+  snprintf (p, 64, "%u", pages);
   SString cs;
   cs.append ("%%Pages: "); 
   cs.append (p); cs.append ("\n");
@@ -468,12 +469,17 @@ SPostscript::cacheOn (bool on)
   cs.append ("%%EndProlog\n\n");
   cs.append ("%%EndProlog\n\n");
   writeString (cs);
-  return SCanvas::cacheOn(on);
+
+  //return SCanvas::cacheOn(on);
+  bool old = isCacheOn;
+  isCacheOn = on;
+  return old;
+
 }
 
 /**
  * start a new path. If cache is on and is is not ""
- * @return true if no more draing is needed.
+ * @return true if no more drawing is needed.
  */
 bool
 SPostscript::beginImage (double _x, double _y, const SString& _id, const SColor& background)
@@ -490,7 +496,7 @@ SPostscript::beginImage (double _x, double _y, const SString& _id, const SColor&
 
   if (!isCacheOn)
   {
-    /* This is a hack. now I requere you to cahce first. */
+    /* This is a hack. now I requere you to cache first. */
     /* for some reason this one was inserting a newpath with no fill */
     // FIXME
     _beginImage (_x, _y, _id);
@@ -513,7 +519,7 @@ SPostscript::newpath ()
       char arr[64];
       int ix =  (int) (1000.0 * _x);
       int iy =  (int) (1000.0 * ((double)height-_y));
-      sprintf (arr, "gsave %s%d.%03d %s%d.%03d translate newpath\n", 
+      snprintf (arr, 64, "gsave %s%d.%03d %s%d.%03d translate newpath\n", 
         SG(ix), ix/1000, SM(ix), SG(iy), iy/1000, SM(iy));
       // BUG-2011-08-04 Don't write. This happens between U+0000...U+001F
       //writeString (arr);
@@ -543,7 +549,7 @@ SPostscript::endImage ()
     if (cache.get (cacheID)) return;
     cache.put (cacheID, cacheCurrent);
     char buff[64];
-    sprintf (buff, "Glyph%u", cacheIDMapCount++);
+    snprintf (buff, 64, "Glyph%u", cacheIDMapCount++);
     cacheIDMap.put (cacheID, SString(buff));
     cacheCurrent.clear();
     cacheID.clear();
@@ -599,7 +605,7 @@ SPostscript::moveto (double _x, double _y)
   char ins[128];
   int ix =  (int) (1000.0 * (x-cacheOriginX));
   int iy =  (int) (1000.0 * (cacheOriginY-y));
-  sprintf (ins, "%s%d.%03d %s%d.%03d moveto\n", 
+  snprintf (ins, 128, "%s%d.%03d %s%d.%03d moveto\n", 
      SG(ix), ix/1000, SM(ix), SG(iy), iy/1000, SM(iy));
   if (isCacheOn)
   {
@@ -621,7 +627,7 @@ SPostscript::lineto (double _x, double _y)
   char ins[128];
   int ix =  (int) (1000.0 * (x-cacheOriginX));
   int iy =  (int) (1000.0 * (cacheOriginY-y));
-  sprintf (ins, "%s%d.%03d %s%d.%03d lineto\n", 
+  snprintf (ins, 128, "%s%d.%03d %s%d.%03d lineto\n", 
      SG(ix), ix/1000, SM(ix), SG(iy), iy/1000, SM(iy));
   if (isCacheOn)
   {
@@ -653,7 +659,7 @@ SPostscript::curveto (double _x0, double _y0, double _x1, double _y1, double _x2
   int iy1 =  (int) (1000.0 * (cacheOriginY-y1));
   int ix2 =  (int) (1000.0 * (x2-cacheOriginX));
   int iy2 =  (int) (1000.0 * (cacheOriginY-y2));
-  sprintf (ins, "%s%d.%03d %s%d.%03d %s%d.%03d %s%d.%03d %s%d.%03d %s%d.%03d curveto\n", 
+  snprintf (ins, 128, "%s%d.%03d %s%d.%03d %s%d.%03d %s%d.%03d %s%d.%03d %s%d.%03d curveto\n", 
      SG(ix0), ix0/1000, SM(ix0), SG(iy0), iy0/1000, SM(iy0), 
      SG(ix1), ix1/1000, SM(ix1), SG(iy1), iy1/1000, SM(iy1), 
      SG(ix2), ix2/1000, SM(ix2), SG(iy2), iy2/1000, SM(iy2));
@@ -729,7 +735,7 @@ SPostscript::bitfont (const SPen& pen, double x, double y,
 {
 #ifndef USE_WINAPI 
 # if USE_X11
-  SX11Impl* impl=(SX11Impl*) SAwt::delegate;
+  SX11Impl* impl=(SX11Impl*) SAwt::getDelegate();
   if (impl == 0)
   {
     fprintf (stderr, "(impl) no luck in prining bitmap font.\n");
@@ -986,3 +992,29 @@ SPostscript::timeout (const SEventSource* s)
   if (timer==0) return false;
   return true;
 }
+
+/*
+bool
+SCanvas::cacheOn(bool on)
+{
+  bool old = isCacheOn;
+  isCacheOn = on;
+  return old;
+}
+*/
+
+SS_Matrix2D
+SPostscript::getCurrentMatrix() const
+{
+  return SS_Matrix2D();
+}
+void
+SPostscript::putImage (int x, int y, const SImage& image)
+{
+}
+void
+SPostscript::setBackground(const SColor &color)
+{
+}
+
+
