@@ -1,7 +1,7 @@
 /**
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 1997-2006  Gaspar Sinai <gaspar@yudit.org>  
+ *  GNU Copyright (C) 1997-2023  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -216,7 +216,7 @@ SScriptProcessor::isSupported (SS_UCS4 c) const
   /* Malayalam */
   if (c >= 0x0d00 && c<= 0x0d7f) return true;
   /* Sinhala - not supported, need to get info */
-  if (c >= 0x0d80 && c<= 0x0dff) return false; //<<<<
+  if (c >= 0x0d80 && c<= 0x0dff) return true; //<<<<
   /* Thai */
   if (c >= 0x0e00 && c<= 0x0e7f) return true;
   /* Lao */
@@ -279,7 +279,7 @@ SScriptProcessor::put (const SS_UCS4* in, unsigned int len, bool is_begin)
 
   unsigned int reallen = 0; // Real length.
   unsigned int min = 0;     // minimum range (inclusive)
-  unsigned int max = 0;     // maxinum range (non-inclusive)
+  unsigned int max = 0;     // maximum range (non-inclusive)
   m_script_has_reph = false;
   m_ra = 0;
   const SRendClass::RType* order = 0; 
@@ -1796,7 +1796,7 @@ SScriptProcessor::reorderIndic ()
       m_out_type.replace (c_last_pos, SRendClass::Clast);
   }
 
-  // Add additional properries for rendering.
+  // Add additional properties for rendering.
 
   unsigned int ucount =0;
   for (i=0; i< m_in.size(); i++)
@@ -1884,7 +1884,7 @@ SScriptProcessor::apply ()
     gsub_guide = SStringVector("nukt,akhn,rphf,blwf,half,pstf,vatu,pres,abvs,blws,psts,haln");
     gpos_guide = SStringVector("abvm,blwm,dist");
     break;
-  // Go with the generic recipie
+  // Go with the generic recipe
   case SC_ORIYA:
   case SC_KANNADA:
   case SC_MALAYALAM:
