@@ -1,7 +1,7 @@
 /** 
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 1997-2006  Gaspar Sinai <gaspar@yudit.org>  
+ *  GNU Copyright (C) 1997-2023  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -39,7 +39,7 @@
 #include "stoolkit/SStringVector.h"
 #include "stoolkit/SBinHashtable.h"
 #include "swindow/SAwt.h"
-#include "swindow/SGEngine.h"
+#include "swindow/SRasterizer.h"
 #include "swindow/swin32/SWin32.h"
 #include "swindow/SRedrawEvent.h"
 
@@ -149,7 +149,7 @@ HINSTANCE pinstance;
 LPSTR cmdLine;
 int cmdShow;
 HANDLE accel;
-/* alway reset it on colormap change */
+/* always reset it on colormap change */
 static HDC compatibleDC = 0;
 static HBITMAP compatibleHBitmap = 0;
 
@@ -294,7 +294,7 @@ private:
 int
 SBitmapArea::put (const SString& key, SBitmapItem* item, SString* old)
 {
-  /* cerate compatible bitmap */
+  /* create compatible bitmap */
   if (!ison) return 0;
   if (!item->dc) return 0;
   if (!item->bitmap) return 0;
@@ -727,7 +727,7 @@ static void lostCapture (long wid);
 static void processMouseWheel (long hwnd, int increment);
 
 /**
- * This is wher ewe start off
+ * This is where we start off
  */
 int APIENTRY
 WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdLine, int nCmdShow)
@@ -831,7 +831,7 @@ WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdLine, int nC
   }
   else
   {
-    fprintf (stderr, "clipboard format errror\n");
+    fprintf (stderr, "clipboard format error\n");
   }
   {
     SWHandler h(msecs, iswine);
@@ -903,6 +903,13 @@ SW32Impl::getWindow (SWindowListener* l, const SString& name)
 SW32Window::SW32Window(const SString& n, SW32Impl* i, long _id) 
   : name(n), background ("white"), pen (SColor(0), SColor(0xffffffff))
 {
+  modalFlag = false;
+  xpos = SD_WIN_X;
+  ypos = SD_WIN_Y;
+  xwidth = SD_WIN_W;
+  yheight = SD_WIN_H;
+
+  isCacheOn = true;
   parentID = 0;
   currentFocusWindow = 0;
   shown = false;
@@ -1022,7 +1029,7 @@ SW32Window::hide ()
 }
 
 /**
- * This reqests a redraw, efficiently after all events got processed.
+ * This requests a redraw, efficiently after all events got processed.
  * @param clear is true if the window needs to be cleared before calling redraw.
  * @param x is the x origin of the event
  * @param y is the y origin of the event
@@ -1306,7 +1313,7 @@ _eventHandler (HWND hwnd, UINT message, WPARAM wParam,LPARAM lParam)
 
           if (w->cdc != 0)
           {
-             //fprintf (stderr, "PAINT: WONT...");
+             //fprintf (stderr, "PAINT: WON'T...");
           }
           else
           {
@@ -1867,7 +1874,7 @@ SW32Window::bitpoints (const SColor& clr, const int* _x, const int* _y,
 }
 
 /**
- * Afetr this size things wont be cached.
+ * After this size things wont be cached.
  */
 void
 SW32Window::setPixmapCacheSize(unsigned int _size)
@@ -1896,7 +1903,7 @@ SW32Window::putImage (int _x, int _y, const SImage& im)
 {
   bool mydc = dcin();
 
-  /* bitmap may be selected in a dc . dont delet now thinking....*/
+  /* bitmap may be selected in a dc . dont delete now thinking....*/
   const SString& ks = (const SString&) im.getID();
   char a[10];
   const SColor& cf = pen.getForeground();
@@ -2003,8 +2010,8 @@ SW32Window::putImage (int _x, int _y, const SImage& im)
 /**
  * Create two bitmaps, one for the image and one for the colored ones.
  * @param p is the image
- * @param m is the shape image. it contains 1's where ther is no image.
- * @param dc is a dc that has the original colored bitmap selcted in it.
+ * @param m is the shape image. it contains 1's where there is no image.
+ * @param dc is a dc that has the original colored bitmap selected in it.
  * @param _dc is a dc we can use - it usually has a 1 depth bitmap in it
  */
 static void
@@ -2067,8 +2074,8 @@ createColoredBitmap (const SPen& pen, const SImage& im,
 /**
  * Create two bitmaps, one for the image and one for the shade.
  * @param p is the image
- * @param m is the shape image. it contains 1's where ther is no image.
- * @param dc is a dc that has the original colored bitmap selcted in it.
+ * @param m is the shape image. it contains 1's where there is no image.
+ * @param dc is a dc that has the original colored bitmap selected in it.
  * @param _dc is a dc we can use - it usually has a 1 depth bitmap in it
  */
 static void
@@ -2108,7 +2115,7 @@ createShadedBitmap (const SPen& pen, const SImage& im,
   COLORREF* colors  = new COLORREF[shades];
   CHECK_NEW (colors);
   /*
-   * We could blend it with the corrent background, 
+   * We could blend it with the current background, 
    * but we would lose a lot of speed...
    */
   for (i=0; i<shades; i++)
@@ -2158,7 +2165,7 @@ bool
 SW32Window::beginImage (double _x, double _y, const SString& _id, const SColor& back)
 {
   pen.setBackground (back);
-  if (engine ==0) engine = new SGEngine();
+  if (engine ==0) engine = new SRasterizer();
   if (isCacheOn)
   {
     return engine->beginImage ((int)_x, (int)_y, _id);
@@ -2202,7 +2209,7 @@ SW32Window::stroke (const SPen& _pen)
 
 
 /**
- * FIXME: fill and return the resulting image for better cahcing.
+ * FIXME: fill and return the resulting image for better caching.
  */
 void
 SW32Window::endImage ()
@@ -2212,7 +2219,7 @@ SW32Window::endImage ()
   SImage* si= engine->endImage ();
   if (si==0) return; /* offscreen */
 
-  /* Use the putimage that does some cacheing. */
+  /* Use the putimage that does some caching. */
   if (imageCache.isOn())
   {
     putImage (si->getOrigoX(), si->getOrigoY(), *si);
@@ -2308,35 +2315,35 @@ SW32Window::closepath()
 void
 SW32Window::rotate (double angle)
 {
-  if (engine ==0) engine = new SGEngine();
+  if (engine ==0) engine = new SRasterizer();
   engine->rotate (angle);
 }
 
 void
 SW32Window::scale (double x, double y)
 {
-  if (engine ==0) engine = new SGEngine();
+  if (engine ==0) engine = new SRasterizer();
   engine->scale (x, y);
 }
 
 void
 SW32Window::translate (double x, double y)
 {
-  if (engine ==0) engine = new SGEngine();
+  if (engine ==0) engine = new SRasterizer();
   engine->translate (x, y);
 }
 
 void
 SW32Window::pushmatrix()
 {
-  if (engine ==0) engine = new SGEngine();
+  if (engine ==0) engine = new SRasterizer();
   engine->pushmatrix();
 }
 
 void
 SW32Window::popmatrix()
 {
-  if (engine ==0) engine = new SGEngine();
+  if (engine ==0) engine = new SRasterizer();
   engine->popmatrix();
 }
 
@@ -2625,9 +2632,9 @@ lostCapture (long wid)
   buttonEnter (wid);
 }
 
-
+/* isPrimary is unused, all is primary */
 SString
-SW32Window::getClipUTF8()
+SW32Window::getClipUTF8(bool isPrimary)
 {
    SString cld;
    bool ucs = true;
@@ -2722,8 +2729,9 @@ SW32Window::getClipUTF8()
    return SString (utf8enc.encode (ucstext));
 }
 
+/* isPrimary is unused, all is primary */
 void
-SW32Window::putClipUTF8(const SString& utf8)
+SW32Window::putClipUTF8(const SString& utf8, bool isPrimary)
 {
   SEncoder utf8enc ("utf-8-s");
   clipData = utf8enc.decode (utf8);
@@ -2791,7 +2799,7 @@ putClipText()
   out.replaceAll ("\342\200\251", "\r\n");
 
   SString nl; nl.append ((char)0);
-  /* I did not invent this - Microsft does it too :( */
+  /* I did not invent this - Microsoft does it too :( */
   out.replaceAll (nl, " ");
 
   out.append ((char)0);
@@ -2850,7 +2858,7 @@ notifyClip ()
 {
   /* Before returning to event loop check if clipboard 
    * owner changed. SetClipboardViewer is a crzay 
-   * mind's creation treat is as non-existant.
+   * mind's creation treat is as non-existent.
    */ 
   HWND owner  = GetClipboardOwner();
   if ((long)owner != clipboardOwner)
@@ -2868,6 +2876,9 @@ notifyClip ()
 void
 SW32Window::setModal (SWindow* _parent, bool decorated)
 {
+  /* CHECKME */
+  modalFlag = true;
+
   modalID = ((SW32Window*)_parent)->getID();
   SetWindowLong ((HWND)id, GWL_STYLE, 
      SS_YUDIT_DIALOG_STYLE);
@@ -3002,7 +3013,7 @@ sendKeyChar (KeyData* kd, const SString& _s)
 }
 
 /**
- * Send release key event to those unfortunatelly windows that
+ * Send release key event to those unfortunately windows that
  * lost keyboard focus as windows does not sent it :(
  */
 static void
@@ -3544,3 +3555,18 @@ SW32Window::isDoubleBufferEnabled () const
 {
   return dbufferOn;
 }
+// Canvas became virtual.
+bool
+SW32Window::cacheOn(bool on)
+{
+  bool old = isCacheOn;
+  isCacheOn = on;
+  return old;
+}
+
+SS_Matrix2D
+SW32Window::getCurrentMatrix() const
+{
+  if (engine!=0) return engine->getCurrentMatrix();
+  return SS_Matrix2D();
+}
