@@ -1,7 +1,7 @@
 /** 
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 1997-2006  Gaspar Sinai <gaspar@yudit.org>  
+ *  GNU Copyright (C) 1997-2023  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -69,8 +69,10 @@ SX11Impl::SX11Impl(void) : in (SEventSource::SOCKET, 0)
   xdndSource = None;
   acceptedDataTypeAtom = None;
   accelPressed = false;
-  clipOwner = None;
-  clipBuffer = "";
+  clipOwnerPrimary = None;
+  clipOwnerSecondary = None;
+  clipBufferPrimary = "";
+  clipBufferSecondary = "";
   encoder = SEncoder("iso-2022-x11");
   display = XOpenDisplay (0);
   job = new SJob();
@@ -81,6 +83,8 @@ SX11Impl::SX11Impl(void) : in (SEventSource::SOCKET, 0)
   }
   else
   {
+    xa_clipboard = XInternAtom (display, "CLIPBOARD", False);
+    xa_property_yudit = XInternAtom (display, "YUDIT_PROPERTY", False);
     XSetErrorHandler (handleX11Errors);
     screen = DefaultScreen (display);
     root = RootWindow (display, screen);
@@ -126,10 +130,14 @@ SX11Impl::SX11Impl(void) : in (SEventSource::SOCKET, 0)
 
 SX11Impl::~SX11Impl()
 {
-  if (clipOwner != None)
+  if (clipOwnerPrimary != None)
   {
     XSetSelectionOwner (display, XA_PRIMARY, None, CurrentTime);
   }
+  if (clipOwnerSecondary != None)
+  {
+    XSetSelectionOwner (display, xa_clipboard, None, CurrentTime);
+  }
   if (display != 0)
   {
     XCloseDisplay (display);
@@ -200,7 +208,7 @@ SX11Impl::doX()
 
   /* block - true */
   {
-    /* we dont process things in a nomral socket event looop. */
+    /* we dont process things in a nomral socket event loop. */
     /* everything is moved to job - done. */
     XEvent event;
     /* NextEvent sucks in all events from socket, and pass the first queued. */
@@ -315,7 +323,7 @@ if (debug && event.type == MapNotify) fprintf(stderr, "MapNotify\n");
       {
          /* add to modal stack, or remove from it */
          unsigned int modalIndex=0;
-         /* earch foir this modal box... */
+         /* search foir this modal box... */
          for (modalIndex=0; modalIndex<modalStack.size(); modalIndex++)
          {
            if (modalStack[modalIndex] == sw->getID())
@@ -397,14 +405,18 @@ if (debug && event.type == MapNotify) fprintf(stderr, "MapNotify\n");
       }
       break;
     case SelectionClear:
+      processSelection (event.xselectionclear);
+      break;
     case SelectionNotify:
+      processSelection (event.xselection);
+      break;
     case SelectionRequest:
-      clipEvent (event);
+      processSelection (event.xselectionrequest);
       break;
     case KeyPress:
     case KeyRelease:
       {
-        /* events that hapen during conversion that dont have sent_event
+        /* events that happen during conversion that dont have sent_event
          * set should be ignored */
 
         /* Don't try to make this bigger because it will screw up 
@@ -508,7 +520,7 @@ if (debug && event.type == MapNotify) fprintf(stderr, "MapNotify\n");
         if (!event.xkey.send_event && sw->isKInput() 
               && sw == keyboardFocusWindow)
         {
-          //fprintf (stderr, "Ignoreing keys.\n");
+          //fprintf (stderr, "Ignoring keys.\n");
           if (event.type==KeyRelease)
           {
             l->keyReleased (sw, skey, s, ctrl, shift, meta);
@@ -528,23 +540,20 @@ if (debug && event.type == MapNotify) fprintf(stderr, "MapNotify\n");
       break;
       
     case ConfigureNotify:
-      if (needFocusWhenMapped == sw->getID())
-      {
-        if (getKeyboardFocus (sw)) needFocusWhenMapped = 0;
-      }
 /*
-{
-bool debug = (sw->getWidth() == 20 && sw->getHeight() == 359);
-debug = debug || (sw->getWidth() == 10 && sw->getHeight() == 200);
-if (debug) fprintf (stderr, "ConfigureNotify %lx: %u,%u\n", 
-(unsigned long) sw, event.xconfigure.width, event.xconfigure.height);
-}
+      if (event.xconfigure.x == 0 &&  event.xconfigure.y == 0) {
+        fprintf (stderr, "ConfigureNotify ZERO %lx  %d,%d\n", 
+            event.xconfigure.window,
+            event.xconfigure.width, event.xconfigure.height);
+      }
 */
-      //if (sw->isChild) break;
+      // FIXME: XMoveWindow does not generate events, something else does.
+      // sw->isChild fix:
+      if (sw->isChild) break;
       sw->setPosition( event.xconfigure.x, event.xconfigure.y);
       sw->setSize( event.xconfigure.width, event.xconfigure.height);
       l->resized (sw, event.xconfigure.x, event.xconfigure.y,
-         event.xconfigure.width, event.xconfigure.height);
+          event.xconfigure.width, event.xconfigure.height);
 
       break;
     case FocusIn:
@@ -566,7 +575,7 @@ if (debug) fprintf (stderr, "ConfigureNotify %lx: %u,%u\n",
       }
       if (keyboardFocusWindow != sw && keyboardFocusWindow!=0)
       {
-        /* wierd eh? we receive a gained focus event, but it is not us! */
+        /* weird eh? we receive a gained focus event, but it is not us! */
 /* 
 Gaspar 2007-12-07 
 Commented out because this can cause an infinite loop.
@@ -767,7 +776,7 @@ void
 SX11Impl::addRedrawChildrenEvent (long id, const SRedrawEvent& evt)
 {
   /* FIXME: finish addRedrawChildrenEvent and get rid of this  */
-  /* this will generate expose event fro child windows. */
+  /* this will generate expose event from child windows. */
 //fprintf (stderr, "addRedrawChildrenEvent\n");
   //XMapSubwindows (display, (Window) id);
 }
@@ -1079,22 +1088,31 @@ SX11Impl::sendString (long id, const SString& s)
  * FIXME
  */
 SString
-SX11Impl::getClipUTF8(long id)
+SX11Impl::getClipUTF8(long id, bool isPrimary)
 {
   Window    owner;
   SEncoder utf8e;
+  // https://www.uninformativ.de/blog/postings/2017-04-02/0/POSTING-en.html
 
   // If we own the selection ourselves, no need for interprocess
   // communication.
-  if (clipOwner==(owner=XGetSelectionOwner (display, XA_PRIMARY)))
-  {
-    return SString(clipBuffer);
-  }
-  clipBuffer.clear();
-  clipOwner = None;
+  Atom  clipBoard = isPrimary ? XA_PRIMARY :  xa_clipboard;
 
+  owner=XGetSelectionOwner (display, clipBoard);
+  if (isPrimary) {
+    if (clipOwnerPrimary == owner) {
+        return SString(clipBufferPrimary);
+    }
+  } else {
+    if (clipOwnerSecondary == owner) {
+        return SString(clipBufferSecondary);
+    }
+  }
+
+  SString& clipBuffer = isPrimary ? clipBufferPrimary : clipBufferSecondary;
+  clipBuffer.clear();
   // Try UTF8_STRING first.
-  clipBuffer = getSelectionData (XA_PRIMARY, toAtom("UTF8_STRING"), id, CurrentTime);
+  clipBuffer = getSelectionData (clipBoard, toAtom("UTF8_STRING"), id, CurrentTime);
   if (clipBuffer.size())
   {
     //fprintf (stderr, "SX11Impl::getClipUTF8: got UTF8_STRING\n");
@@ -1102,7 +1120,7 @@ SX11Impl::getClipUTF8(long id)
   }
 
   // Then try COMPOUND_TEXT.
-  clipBuffer = getSelectionData (XA_PRIMARY, toAtom("COMPOUND_TEXT"), id, CurrentTime);
+  clipBuffer = getSelectionData (clipBoard, toAtom("COMPOUND_TEXT"), id, CurrentTime);
   if (clipBuffer.size())
   {
     //fprintf (stderr, "SX11Impl::getClipUTF8: got COMPOUND_TEXT\n");
@@ -1111,7 +1129,7 @@ SX11Impl::getClipUTF8(long id)
   }
 
   // Finally try STRING.
-  clipBuffer = getSelectionData (XA_PRIMARY, XA_STRING, id, CurrentTime);
+  clipBuffer = getSelectionData (clipBoard, XA_STRING, id, CurrentTime);
   if (clipBuffer.size())
   {
     //fprintf (stderr, "SX11Impl::getClipUTF8: got XA_STRING\n");
@@ -1153,12 +1171,9 @@ SX11Impl::getSelectionData (Atom dataName, Atom dataType, long id, Time _time)
   XEvent    xevent;
 
   time_t  start;
-  Atom    propty;
-
-  propty = toAtom ("YUDIT_SELECTION");;
 
   XConvertSelection (display, dataName, dataType, 
-    propty, (Window)id, _time );
+    xa_property_yudit, (Window)id, _time );
 
   XFlush (display);
   for (start = time (0); ; )
@@ -1174,7 +1189,7 @@ SX11Impl::getSelectionData (Atom dataName, Atom dataType, long id, Time _time)
       return SString();
     }
   }
-  return getPropertyData (xevent.xselection.requestor, propty, 8);
+  return getPropertyData (xevent.xselection.requestor, xa_property_yudit, 8);
 }
 
 /**
@@ -1223,29 +1238,42 @@ SX11Impl::getPropertyData (Window window, Atom property, int format)
  * This is using the XA_PRIMARY selection.
  */
 void
-SX11Impl::putClipUTF8 (long id, const SString& utf8)
+SX11Impl::putClipUTF8 (long id, const SString& utf8, bool isPrimary)
 {
   //fprintf (stderr, "FIXME: putClipUTF8\n");
 
   SWindowListener* l;
   SX11Window* sw;
-  if (clipOwner != (Window) id && clipOwner != None)
+  Atom  clipBoard = isPrimary ? XA_PRIMARY : xa_clipboard;
+
+  if (clipOwnerPrimary != (Window) id && clipOwnerPrimary != None)
   {
-    l = listenerHashtable.get((long)clipOwner);
-    sw = (SX11Window*) windowHashtable.get((long)clipOwner);
+    l = listenerHashtable.get((long)clipOwnerPrimary);
+    sw = (SX11Window*) windowHashtable.get((long)clipOwnerPrimary);
     if (l && sw)
     {
       l->lostClipSelection (sw);
     }
   }
   l = listenerHashtable.get((long)id);
-  sw = (SX11Window*) windowHashtable.get((long)clipOwner);
-  clipOwner = None;
-  XSetSelectionOwner (display, XA_PRIMARY, (Window) id, CurrentTime);
-  if (XGetSelectionOwner (display, XA_PRIMARY) != (Window) id)
+  sw = 0;
+
+  if (isPrimary) {
+    sw = (SX11Window*) windowHashtable.get((long)clipOwnerPrimary);
+    clipOwnerPrimary = None;
+  } else {
+    sw = (SX11Window*) windowHashtable.get((long)clipOwnerSecondary);
+    clipOwnerSecondary = None;
+  }
+  XSetSelectionOwner (display, clipBoard, (Window) id, CurrentTime);
+  if (XGetSelectionOwner (display, clipBoard) != (Window) id)
   {
     /* Oops */
-    clipOwner = None;
+    if (isPrimary) {
+        clipOwnerPrimary = None;
+    } else {
+        clipOwnerSecondary = None;
+    }
     if (l && sw)
     {
       fprintf (stderr, "XGetSelectionOwner oops\n");
@@ -1253,56 +1281,47 @@ SX11Impl::putClipUTF8 (long id, const SString& utf8)
     }
     return;
   }
-  clipOwner = (Window) id;
-  clipBuffer = utf8;
+  if (isPrimary) {
+    clipOwnerPrimary = (Window) id;
+    clipBufferPrimary = utf8;
+  } else {
+    clipOwnerSecondary = (Window) id;
+    clipBufferSecondary = utf8;
+  }
 }
 
-/**
- * process the following events:
- * <ul>
- * <li> SelectionClear </li>
- * <li> SelectionNotify </li>
- * <li> SelectionRequest </li>
- * <ul>
- * @param event is the event to process.
- */
+// https://www.x.org/releases/X11R7.5/doc/man/man3/XSelectionRequestEvent.3.html
 void
-SX11Impl::clipEvent (const XEvent& event)
+SX11Impl::processSelection (const XSelectionRequestEvent& sel)
 {
-  SWindowListener* l = listenerHashtable.get((long)clipOwner);
-  SX11Window* sw = (SX11Window*) windowHashtable.get((long)clipOwner);
-
-  XSelectionRequestEvent sel = event.xselectionrequest;
-
   XEvent   xevent;
   SEncoder utf8e;
   SString  str;
 
-  if (sw == 0 || sw->getID() != (long)event.xany.window)
+  SString& clipBuffer = clipBufferPrimary;
+  SX11Window* sw = 0;
+  if (sel.selection == xa_clipboard) 
   {
-    fprintf (stderr, "oops SX11Impl::clipEvent - clear.\n");
+    sw = (SX11Window*) windowHashtable.get((long)clipOwnerSecondary);
+    clipBuffer = clipBufferSecondary;
+  }
+  else if (sel.selection == XA_PRIMARY) 
+  {
+    sw = (SX11Window*) windowHashtable.get((long)clipOwnerPrimary);
+    clipBuffer = clipBufferPrimary;
+  } else {
+        fprintf (stderr, "Unknown selection in XSelectionRequestEvent\n");
+        return;
+  }
+  if (sw == 0 || sw->getID() != (long)sel.owner)
+  {
+    fprintf (stderr, "OOPS XSelectionRequestEvent.\n");
     return;
   }
-
-  switch (event.type)
+  /* FIXME: This does not answer MULTIPLE and TIMESTAMP requests. */
+  Atom property = sel.property;
+  if (sel.target == XInternAtom(display,"TARGETS",False)) 
   {
-  case SelectionClear:
-//fprintf (stderr, "clipevent - clear.\n");
-    if (l && sw)
-    {
-      l->lostClipSelection (sw);
-    }
-    clipOwner = None;
-    return;
-  case SelectionNotify:
-//fprintf (stderr, "clipevent - notify.\n");
-    clipOwner = None;
-    return;
-  case SelectionRequest:
-    /* FIXME: This does not answer MULTIPLE and TIMESTAMP requests. */
-    if (sel.target == XInternAtom(display,"TARGETS",False))
-    {
-//fprintf (stderr, "clipevent - targets.\n");
       Atom* targets = new Atom[5];
       targets[0] = XInternAtom(display,"TARGETS",False);
       targets[1] = XInternAtom(display,"TEXT",False);
@@ -1348,19 +1367,67 @@ SX11Impl::clipEvent (const XEvent& event)
     }
     else
     {
-       sel.property = None;
+       property = None;
     }
     xevent.xselection.type = SelectionNotify;
     xevent.xselection.display = sel.display;
     xevent.xselection.requestor = sel.requestor;
     xevent.xselection.selection = sel.selection;
     xevent.xselection.target = sel.target;
-    xevent.xselection.property = sel.property;
+    xevent.xselection.property = property;
     xevent.xselection.time = sel.time;
     XSendEvent (sel.display, sel.requestor, False, 0, &xevent);
-    break;
-  default:
-    break;
+}
+
+// https://www.x.org/releases/X11R7.5/doc/man/man3/XSelectionEvent.3.html
+void
+SX11Impl::processSelection (const XSelectionEvent& sel)
+{
+// Ignore
+#if 0
+  if (sel.selection == xa_clipboard) 
+  {
+    clipOwnerSecondary = None;
+  }
+  else if (sel.selection == XA_PRIMARY) 
+  {
+    clipOwnerPrimary = None;
+  } else {
+        fprintf (stderr, "Unknown selection in XSelectionEvent\n");
+        return;
+  }
+#endif
+}
+
+// https://www.x.org/releases/X11R7.5/doc/man/man3/XSelectionClearEvent.3.html
+void
+SX11Impl::processSelection (const XSelectionClearEvent& sel)
+{
+  SWindowListener* l = 0;
+  SX11Window* sw = 0;
+  if (sel.selection == xa_clipboard) 
+  {
+      l = listenerHashtable.get((long)clipOwnerSecondary);
+      sw = (SX11Window*) windowHashtable.get((long)clipOwnerSecondary);
+      clipOwnerSecondary = None;
+  }
+  else if (sel.selection == XA_PRIMARY) 
+  {
+      l = listenerHashtable.get((long)clipOwnerPrimary);
+      sw = (SX11Window*) windowHashtable.get((long)clipOwnerPrimary);
+      clipOwnerPrimary = None;
+  } else {
+      fprintf (stderr, "Unknown selection in XSelectionClearEvent\n");
+      return;
+  }
+  if (sw == 0 || sw->getID() != (long)sel.window)
+  {
+    fprintf (stderr, "OOPS XSelectionClearEvent.\n");
+    return;
+  }
+  if (l && sw)
+  {
+    l->lostClipSelection (sw);
   }
 }
 
@@ -1573,7 +1640,7 @@ SX11Impl::xdndEnter (const XEvent& event)
  * interested in, and the given time stamp. (7) It can do this more than once, 
  * if necessary. 
  *
- * If it can accept the drop, it should hilight its border to notify the user. 
+ * If it can accept the drop, it should highlight its border to notify the user. 
  * If it retrieved the data, it should cache it so it does not need to
  * be retrieved again when the actual drop occurs. 
  * ----------------------------------------------------------------------------
@@ -1697,7 +1764,7 @@ SX11Impl::xdndDrop (const XEvent& event)
     //fprintf (stderr, "got data: [%*.*s]\n", SSARGS(data));
   }
      
-  /* create an xdnd finsished */
+  /* create an xdnd finished */
   XEvent cevt;
   cevt.xclient.type = ClientMessage;
   cevt.xclient.window = xdndSource;
