@@ -1,7 +1,7 @@
 /** 
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 1997-2006  Gaspar Sinai <gaspar@yudit.org>  
+ *  GNU Copyright (C) 1997-2023  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -42,7 +42,7 @@ static unsigned int cacheCount = 0;
 static bool needClear = false;
 
 /**
- * Afetr this size things wont be cached.
+ * After this size things wont be cached.
  */
 void
 SX11Window::setPixmapCacheSize(unsigned int size)
@@ -59,10 +59,14 @@ SX11Window::setPixmapCacheOn (bool on)
   pixmapCacheOn=on;
   needClear = false;
 }
+/* 
+  SGC MAKKA StructureNotifyMask may collide with SubstructureNotifyMask
+  for ConfigureNotify   
+  SubstructureNotifyMask \
+*/
 
 #define SD_YUDIT_EVENT_MASK ( ExposureMask \
   | StructureNotifyMask \
-  | SubstructureNotifyMask \
   | ButtonPressMask \
   | ButtonReleaseMask \
   | Button1MotionMask \
@@ -81,6 +85,13 @@ SX11Window::SX11Window(const SString& n, SX11Impl* _impl, long _id) : name (n),
  background (SX11Color(_impl, SS_WORD32(0xffffffff))),
  pen (SColor(0), SColor(0xffffffff))
 {
+  modalFlag = false;
+  xpos = SD_WIN_X;
+  ypos = SD_WIN_Y;
+  xwidth = SD_WIN_W;
+  yheight = SD_WIN_H;
+
+  isCacheOn = true;
   impl = _impl;
   dbindex = 0;
 
@@ -157,7 +168,7 @@ SX11Window::beginImage (double _x, double _y, const SString& _id, const SColor&
     /* this will always return false */
     return engine->beginImage ((int)_x, (int)_y, "");
   }
-  if (engine ==0) engine = new SGEngine();
+  if (engine ==0) engine = new SRasterizer();
   if (isCacheOn)
   {
     return engine->beginImage ((int)_x, (int)_y, _id);
@@ -188,7 +199,7 @@ SX11Window::fill (const SPen& _pen) {
 #define SS_DOUBLE_SCAN 1
 
 /**
- * fill and return the resulting image for better cahcing.
+ * fill and return the resulting image for better caching.
  */
 void
 SX11Window::endImage ()
@@ -202,7 +213,7 @@ SX11Window::endImage ()
   SImage* si= engine->endImage ();
   if (si==0) return; /* offscreen */
 
-  /* Use the putimage that does some cahcheing. */
+  /* Use the putimage that does some caching. */
   if (pixmapCacheOn)
   {
     putImage (si->getOrigoX(), si->getOrigoY(), *si);
@@ -226,7 +237,7 @@ SX11Window::endImage ()
   
   int i;
   /*
-   * We could blend it with the corrent background, 
+   * We could blend it with the current background, 
    * but we would lose a lot of speed...
    */
   for (i=0; i<shades; i++)
@@ -359,7 +370,7 @@ SX11Window::closepath()
 /**
  * put an image to the scene. originX and OriginY is ignored.
  * This version is using full pixmaps for speed.
- * @param x is the x corener of the image.
+ * @param x is the x corner of the image.
  * @param y is the y corner of the image.
  * @param image is the actual image.
  */
@@ -373,8 +384,8 @@ SX11Window::putImage (int _x, int _y, const SImage& im)
   }
   if (cacheCount > cacheSize)
   {
-      fprintf (stderr, "SX11Window:: clearing cache (%u elements)\n",
-          cacheCount);
+      //fprintf (stderr, "SX11Window:: clearing cache (%u elements)\n",
+      //    cacheCount);
       needClear = true;
   }
   if (needClear && cacheCount!=0)
@@ -568,7 +579,7 @@ SX11Window::createColoredPixmap (const SImage& im, Pixmap* p, Pixmap* m)
  * Create two pixmaps, one for the image and one for the shade.
  * We use the same depth to avoid XCopyPlanes to gain speed
  * @param p is the image
- * @param m is the shape image. it contains 1's where ther is no image.
+ * @param m is the shape image. it contains 1's where there is no image.
  */
 void
 SX11Window::createShadedPixmap (const SImage& im, Pixmap* p, Pixmap* m)
@@ -616,7 +627,7 @@ SX11Window::createShadedPixmap (const SImage& im, Pixmap* p, Pixmap* m)
   SX11Color** colors  = new SX11Color*[shades];
   CHECK_NEW (colors);
   /*
-   * We could blend it with the corrent background, 
+   * We could blend it with the current background, 
    * but we would lose a lot of speed...
    */
   for (i=0; i<shades; i++)
@@ -756,14 +767,14 @@ SX11Window::setBackground(const SColor &color)
 void
 SX11Window::rotate (double angle)
 {
-  if (engine ==0) engine = new SGEngine();
+  if (engine ==0) engine = new SRasterizer();
   engine->rotate (angle);
 }
 
 void
 SX11Window::scale (double x, double y)
 {
-  if (engine ==0) engine = new SGEngine();
+  if (engine ==0) engine = new SRasterizer();
 //fprintf (stderr, "scaling %g %g\n", x, y);
   engine->scale (x, y);
 }
@@ -771,31 +782,35 @@ SX11Window::scale (double x, double y)
 void
 SX11Window::translate (double x, double y)
 {
-  if (engine ==0) engine = new SGEngine();
+  if (engine ==0) engine = new SRasterizer();
   engine->translate (x, y);
 }
 
 void
 SX11Window::pushmatrix()
 {
-  if (engine ==0) engine = new SGEngine();
+  if (engine ==0) engine = new SRasterizer();
   engine->pushmatrix();
 }
 
 void
 SX11Window::popmatrix()
 {
-  if (engine ==0) engine = new SGEngine();
+  if (engine ==0) engine = new SRasterizer();
   engine->popmatrix();
 }
 
+//extern const char* debugComponent;
+
 void
 SX11Window::resize (unsigned int _width, unsigned int _height)
 {
   //fprintf (stderr, "resizing from: %lx   %u %u to %u %u\n",
   //(unsigned long) this, xwidth, yheight, _width, _height);
+//if (debugComponent) fprintf (stderr, "XResizeWindow %lx: %s %d,%d -> %d,%d\n", 
+ //   getWindowID(), debugComponent, getWidth(), getHeight(), _width, _height);
 
-  if (getWidth() == _width && getHeight() == _height)return;
+  // SGC resize it anyvay if (getWidth() == _width && getHeight() == _height) return;
   if (_width == 0) _width = 1;
   if (_height == 0) _height = 1;
   XResizeWindow (impl->display, (Window) dbuffer[0].id, _width, _height);
@@ -805,7 +820,17 @@ SX11Window::resize (unsigned int _width, unsigned int _height)
 void
 SX11Window::move (int _x, int _y)
 {
-  if (getPositionX() == _x && getPositionY() == _y)return;
+//if (debugComponent) fprintf (stderr, "XMoveWindow %lx: %s %d,%d -> %d,%d\n", 
+ //   getWindowID(), debugComponent, getPositionX(), getPositionY(), _x, _y);
+
+  // Commented because XMoveWindow sometimes fails. 
+  if (getPositionX() == _x && getPositionY() == _y) return;
+/*
+if (getHeight() == 65) {
+  fprintf (stderr, "SGC XMoveWindow %lx  size-%d,%d  xy= %d,%d\n", 
+    getWindowID(), getWidth(), getHeight(), _x, _y);
+}
+*/
   XMoveWindow (impl->display, (Window) dbuffer[0].id, _x, _y);
   setPosition(_x, _y);
 }
@@ -1023,7 +1048,7 @@ SX11Window::getID ()
 }
 
 /**
- * Assign a rectangualr clip area. Everithing outside this area will be clipped.
+ * Assign a rectangualr clip area. Everything outside this area will be clipped.
  */
 void
 SX11Window::setClippingArea (int _x, int _y, unsigned int _width, unsigned int _height)
@@ -1096,7 +1121,7 @@ SX11Window::setApplicationImage (const SImage& im)
 }
 
 /**
- * This reqests a redraw, efficiently after all events got processed.
+ * This requests a redraw, efficiently after all events got processed.
  * This is called by the client and not by SX11Impl.
  * @param clear is true if the window needs to be cleared before calling redraw.
  * @param x is the x origin of the event
@@ -1188,24 +1213,24 @@ SX11Window::isVisible ()
   RETIRED 2001-11-21
 
 /**
- * draw a poligon on the screen with bits.
+ * draw a polygon on the screen with bits.
  */
 void
-SX11Window::bitpolygon (const SColor& fg, const SV_INT& poligon)
+SX11Window::bitpolygon (const SColor& fg, const SV_INT& polygon)
 {
   if (gcFG != (unsigned long)  fg.getValue())
   {
     XSetForeground (impl->display, gc, SX11Color(impl, fg).getPixelValue());
     gcFG = (unsigned long)  fg.getValue();
   }
-  XPoint* points = new XPoint[poligon.size()/2];
-  for (unsigned int i=0; i<poligon.size(); i=i+2)
+  XPoint* points = new XPoint[polygon.size()/2];
+  for (unsigned int i=0; i<polygon.size(); i=i+2)
   {
-    points[i/2].x = poligon[i];
-    points[i/2].y = poligon[i+1];
+    points[i/2].x = polygon[i];
+    points[i/2].y = polygon[i+1];
   }
 
-  XFillPolygon (impl->display, (Window) id, gc, points, poligon.size()/2, 
+  XFillPolygon (impl->display, (Window) id, gc, points, polygon.size()/2, 
      Convex, CoordModeOrigin);
   delete [] points;
 }
@@ -1216,18 +1241,18 @@ SX11Window::bitpolygon (const SColor& fg, const SV_INT& poligon)
  * Get an utf8 encoded text from clipboard.
  */
 SString
-SX11Window::getClipUTF8()
+SX11Window::getClipUTF8(bool isPrimary)
 {
-  return impl->getClipUTF8 (dbuffer[0].id);
+  return impl->getClipUTF8 (dbuffer[0].id, isPrimary);
 }
 
 /**
  * put and utf8-encoded text to clipboard.
  */
 void
-SX11Window::putClipUTF8(const SString& utf8)
+SX11Window::putClipUTF8(const SString& utf8, bool isPrimary)
 {
-  impl->putClipUTF8 (dbuffer[0].id, utf8);
+  impl->putClipUTF8 (dbuffer[0].id, utf8, isPrimary);
 }
 
 /**
@@ -1282,7 +1307,9 @@ SX11Window::setTitle (const SString& title)
 bool
 SX11Window::cacheOn (bool on)
 {
-  return SCanvas::cacheOn(on);
+  bool old = isCacheOn;
+  isCacheOn = on;
+  return old;
 }
 
 /**
@@ -1353,7 +1380,8 @@ void
 SX11Window::setModal (SWindow* _parent, bool decorated)
 {
   XSetWindowAttributes winattr;
-  SWindow::setModal (_parent, decorated);
+  /* CHECKME */
+  modalFlag = true;
   winattr.override_redirect = True;
   winattr.save_under = True;
   if (_parent == 0)
@@ -1474,7 +1502,6 @@ SX11Window::getWindowID() const
 void
 SX11Window::setDoubleBuffer (bool isOn)
 {
-  
   dbindex = isOn ? 1 : 0;
 }
 
@@ -1502,6 +1529,7 @@ SX11Window::freeDoubleBuffer ()
 void
 SX11Window::beginDoubleBuffer (const SX11Region& region)
 {
+  //fprintf (stderr, "beginDoubleBuffer %d\n", dbindex);
   if (dbindex == 0) return;
   // Free Up Previous pixmap if 
   if (dbuffer[1].id)
@@ -1560,6 +1588,7 @@ SX11Window::beginDoubleBuffer (const SX11Region& region)
 void
 SX11Window::endDoubleBuffer (const SX11Region& region)
 {
+  //fprintf (stderr, "endDoubleBuffer %d\n", dbindex);
   if (dbindex == 0)
   {
      if (dbuffer[1].id != None) freeDoubleBuffer ();
