@@ -1,7 +1,7 @@
 /** 
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 1997-2006  Gaspar Sinai <gaspar@yudit.org>  
+ *  GNU Copyright (C) 1997-2023  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -33,7 +33,7 @@
 
 #define BUFFER_SIZE 512
 
-static const char *version="uniconv version %s GNU(c) Gaspar Sinai\n";
+static const char *version="uniconv version %s GNU(C) Gaspar Sinai\n";
 static void usage ();
 
 #define SS_UNIX 0
@@ -143,104 +143,36 @@ main (int argc, char *argv[])
   unixn.append((SS_UCS4) '\n');
   SV_UCS4 macr;
   macr.append((SS_UCS4) '\r');
-  int crpending = 0;
   while (!feof(input) && (len=fread(buff, 1, BUFFER_SIZE, input))>0)
   {
-    if (buff[len-1] == '\r')
-    {
-      crpending = 1;
-      len--;
-      if (!len) break;
-    }
     SString s(buff, len);
     remaining.append(s);
-    /* this will not work if last line does not have a proper ending */
-    while (true)
-    {
-      int found =-1;
-      unsigned int i;
-      for (i=0; i<delimiters.size(); i++)
-      {
-         found = remaining.find (delimiters[i]);
-         if (found>=0) break;
-      }
-      if (found < 0) break;
-      unsigned int ind = found+delimiters[i].size();
-      SString a(remaining.array(), ind);
-      lines.append (a);
-      remaining.remove (0, ind);
-    }
-    for (unsigned k=0; k<lines.size(); k++)
-    {
-      SV_UCS4 res = ic.decode (lines[k]);
-      if (fromd == SS_DOS)
-      {
-        res.replaceAll(dosrn, unixn);
-      }
-      if (fromd == SS_MAC)
-      {
-        res.replaceAll(macr, unixn);
-      }
-      if (tod == SS_MAC)
-      {
-        res.replaceAll(unixn, macr);
-      }
-      if (tod == SS_DOS)
-      {
-        res.replaceAll(dosrn, unixn);
-        res.replaceAll(unixn, dosrn);
-      }
-      if (res.size() != 0)
-      {
-        SString w = oc.encode (res);
-        int wlen = fwrite(w.array(), 1, w.size(), output);
-        if (wlen!= (int) w.size())
-        {
-          fprintf (stderr, "uniconv: failed to write %s.\n", ofile);
-          delete[] buff;
-          return 1;
-        }
-      }
-    }
-    lines.clear();
-    if (crpending)
-    {
-      remaining.append("\r");
-      crpending = 0;
-    }
   }
-  if (crpending)
+  SV_UCS4 res = ic.decode (remaining, false);
+  if (fromd == SS_DOS)
   {
-    remaining.append("\r");
+    res.replaceAll (dosrn, unixn);
   }
-  if (remaining.size() != 0)
+  if (fromd == SS_MAC)
   {
-    SV_UCS4 res = ic.decode (remaining, false);
-    if (fromd == SS_DOS)
-    {
-      res.replaceAll (dosrn, unixn);
-    }
-    if (fromd == SS_MAC)
-    {
-      res.replaceAll(macr, unixn);
-    }
-    if (tod == SS_MAC)
-    {
-      res.replaceAll(unixn, macr);
-    }
-    if (tod == SS_DOS)
-    {
-      res.replaceAll (dosrn, unixn);
-      res.replaceAll (unixn, dosrn);
-    }
-    SString w = oc.encode (res);
-    int wlen = fwrite(w.array(), 1, w.size(), output);
-    if (wlen!= (int) w.size())
-    {
-      fprintf (stderr, "uniconv: failed to write %s [%d/%d].\n", ofile, wlen, w.size());
-      delete[] buff;
-      return 1;
-    }
+    res.replaceAll(macr, unixn);
+  }
+  if (tod == SS_MAC)
+  {
+    res.replaceAll(unixn, macr);
+  }
+  if (tod == SS_DOS)
+  {
+    res.replaceAll (dosrn, unixn);
+    res.replaceAll (unixn, dosrn);
+  }
+  SString w = oc.encode (res);
+  int wlen = fwrite(w.array(), 1, w.size(), output);
+  if (wlen!= (int) w.size())
+  {
+    fprintf (stderr, "uniconv: failed to write %s [%d/%d].\n", ofile, wlen, w.size());
+    delete[] buff;
+    return 1;
   }
   fclose (input);
   fclose (output);
@@ -254,11 +186,7 @@ usage ()
   SString pathStr;
   SStringVector l = SUniMap::getPath();
   
-#ifdef USE_WINAPI
-  pathStr = l.join (";");
-#else
-  pathStr = l.join (":");
-#endif
+  pathStr = l.join (",");
 
   SStringVector enc = SEncoder::builtin();
   SString supStr = enc.join(", ");
@@ -273,7 +201,7 @@ usage ()
   fprintf (stdout, "%*.*s\n\n", SSARGS(supStr));
   fprintf (stdout, "Currently known external encodings:   \n");
   fprintf (stdout, "%*.*s\n\n", SSARGS(supExt));
-  fprintf (stdout, "datapath settings:\n %*.*s.\n\n", SSARGS(pathStr));
+  fprintf (stdout, "datapath settings:\n%*.*s\n\n", SSARGS(pathStr));
   fprintf (stdout, "example: /uniconv -decode Hungarian -encode iso-8859-2\n");
   fprintf (stdout, "example: Sinai Ga'spa'r <eof>\n");
 }
