@@ -1,7 +1,7 @@
 /** 
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 1997-2006  Gaspar Sinai <gaspar@yudit.org>  
+ *  GNU Copyright (C) 1997-2023  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -32,6 +32,7 @@
 #include <swindow/SScriptProcessor.h>
 #include <swindow/SPrinter.h>
 #include <swidget/SUniPrint.h>
+#include <stoolkit/SIO.h>
 
 #include <stdio.h>
 #include <string.h>
@@ -39,8 +40,8 @@
 
 #define BUFFER_SIZE 512
 
-static const char *version="uniprint version %s GNU(c) Gaspar Sinai\n";
-static void usage ();
+static const char *version="uniprint version %s GNU(C) Gaspar Sinai\n";
+static void usage (const SStringVector& fonts);
 
 #define SS_UNIX 0
 #define SS_DOS 1
@@ -73,6 +74,7 @@ main (int argc, char *argv[])
   bool wrap = false;
   double headerSize = DEFAULT_HEADER;
   double bodySize = DEFAULT_BODY;
+  bool is_usage = false;
   
   SUniMap::guessPath();
   SFontImpl::guessPath();
@@ -195,21 +197,24 @@ main (int argc, char *argv[])
     }
     else if (argv[i][0] == '-')
     {
-      usage();
-      return 1;
+      is_usage = 1;
     }
     else 
     {
       if (i+1==argc)
       {
         ifile = argv[i];
-        //fprintf (stderr, "only 1 file will get printed: %s\n", ifile);
         break;
       }
-      usage();
-      return 1;
+      is_usage = true;
+      break;
     }
   }
+  if (is_usage) 
+  {
+    usage (font);
+    return 1;
+  }
 
   /* Sub processing */
 
@@ -274,34 +279,71 @@ main (int argc, char *argv[])
   /**
    * FIXME: move this to yudit config.
    */
-  if (font.size() ==0)
+  SString fontSet("default");
+  bool isFontSet = false;
+  if (font.size() == 1 && font[0].find (".") < 0) {
+    isFontSet = true;
+    fontSet = font[0];
+  }
+  if (isFontSet || font.size() == 0)
   {
     SProperties p;
-    loadProperties(&p);
-    if (p.get("yudit.font.TrueType"))
+    if (fontSet == "default") 
     {
-      SStringVector vf (p["yudit.font.TrueType"]);
-      fprintf (stderr, "No fonts were specified trying property `yudit.font.TrueType'.\n");
+        p.put(fontSet, SFont::getDefaultFontList());
+    }
+    else
+    {
+        SString fs = ("yudit.font.");
+        fs.append (fontSet);
+        fontSet = fs;
+        loadProperties(&p);
+    }
+    font.clear();
+    if (p.get(fontSet))
+    {
+      SStringVector vf (p[fontSet]);
       for (unsigned int m=0; m<vf.size(); m++)
       {
-        SStringVector nvf(vf[m], ":");
-        font.append (nvf[0]);
-        fprintf (stderr, "No fonts were specified - adding %*.*s\n", 
-            SSARGS(nvf[0]));
+        SString f = vf[m];
+        font.append (f);
       }
     }
     if (font.size() ==0)
     {
-      fprintf (stderr, "No fonts were specified - adding cyberbit.ttf,yudit.ttf:cp-1250\n");
-      font.append ("cyberbit.ttf");
-      font.append ("yudit.ttf:cp-1250");
+      fprintf (stderr, "FontSet %*.*s not found. Adding: yudit.ttf,OldHungarian_Full.ttf\n", SSARGS(fontSet));
+      font.append ("yudit.ttf");
+      font.append ("OldHungarian_Full.ttf");
     }
   }
   SFontImplVector himpl;
   SFontImplVector dimpl;
   for (unsigned int h=0; h<font.size(); h++)
   {
-    SFontImpl impl (font[h], font[h]);
+    SString fntfull = font[h];
+    SStringVector v(fntfull, ":", false);
+    SString fnt = v[0];
+    SString enc = "";
+    if (v.size()>1 && v[1].size()!=0) enc = v[1];
+    SFontImpl impl (fnt, enc);  
+    if (v.size()>2 && v[2].size()>0)
+    {
+         SStringVector pvect(v[2], ";");
+         SProperties props;
+         for (unsigned int j=0; j<pvect.size(); j++)
+         {
+            SStringVector vv(pvect[j], "=", true);
+            if (vv.size() > 1)
+            {  
+               props.put (vv[0], vv[1]);
+            }
+            else
+            {
+               props.put (vv[0], "true");
+            }
+         }
+         impl.setAttributes(props);
+    }
     himpl.append (impl);
     dimpl.append (impl);
   }
@@ -336,14 +378,65 @@ main (int argc, char *argv[])
 }
 
 static void
-usage ()
+usage (const SStringVector& font)
 {
-  fprintf (stdout, version, SD_YUDIT_VERSION);
-  fprintf (stdout,   "usage: uniprint [-out file] [-printer name] [-in file] [-decode name] \n");
-  fprintf (stdout,   "[-media A4] [-break] [-right|-left] [-hsize header-font-size] [-size font-size]\n");
-  fprintf (stdout,   "[-L|-landscape] [-wrap] [-font truetype.ttf [-font otherfont.ttf [...]] \n\n");
-  fprintf (stdout,   "uniprint [-us] using uniscribe-clone rendering (default).\n");
-  fprintf (stdout,   "uniprint [-nus] not using uniscribe-clone rendering.\n");
-  fprintf (stdout,   "uniconv -h will get list of supported encodings.   \n");
+  SStringVector l = SFontImpl::getPath();
+  SString pathStr = l.join (",");
+  SString fontList = font.join (",");
+
+  SString fontSet = "indivifual fonts";
+  if (font.size() == 0 || (font.size() == 1 && font[0] == "default")) {
+     fontList = SFont::getDefaultFontList();
+     fontSet = "yudit 'default' font-set";  
+  }
+  if (font.size() == 1 && font[0].find (".") < 0) 
+  {
+       fontSet = "yudit font-set '";
+       fontSet.append(font[0]);
+       fontSet.append("'");
+       SString fs = ("yudit.font.");
+       fs.append (font[0]);
+       SProperties p;
+       loadProperties(&p);
+       if (p.get(fs)) {
+           fontList = p[fs];
+       } else {
+           fontList.append (" - not defined in yudit.properties");
+        }
+   }
+   fprintf (stdout, version, SD_YUDIT_VERSION);
+   fprintf (stdout,   "usage: uniprint [-out file] [-printer name] [-in file] [-decode name] \n");
+   fprintf (stdout,   "[-media A4] [-break] [-right|-left] [-hsize header-font-size] [-size font-size]\n");
+   fprintf (stdout,   "[-L|-landscape] [-wrap] [-font truetype.ttf | YuditFontSet]] \n\n");
+   fprintf (stdout,   "uniprint [-us] using uniscribe-clone rendering (default).\n");
+   fprintf (stdout,   "uniprint [-nus] not using uniscribe-clone rendering.\n");
+   fprintf (stdout,   "Only one Yudit font set or multiple fonts can be spacified.\n\n");
+   fprintf (stdout,   "%*.*s:\n%*.*s\n\n", SSARGS(fontSet), SSARGS(fontList));
+
+   SStringVector fonts(fontList, ",");
+   SStringVector path(pathStr, ",");
+   unsigned int count = 0;
+   for (unsigned int i=0; i<fonts.size(); i++) 
+   {
+      SString fntfull = fonts[i];
+      SStringVector v(fntfull, ":", false);
+      SString fnt = v[0];
+      SFile file(fnt, path);
+      if (file.size() > 0)
+      {
+        SString fname = file.getName();
+        if (fname.find(" ") >= 0) 
+        {
+           fname = SString ("'"); 
+           fname.append (file.getName());
+           fname.append ("'");
+        }
+        fprintf (stdout,   "%*.*s: %*.*s\n", SSARGS(fnt), SSARGS(fname));
+        count++;
+      }
+   }
+   fprintf (stderr, "%d font file(s) found out of %d.\n", count, fonts.size());
+   fprintf (stdout,   "fontpath settings:\n%*.*s\n", SSARGS(pathStr)); 
+      
 }
 
